/**
 * Esimerkki dynaamisesta taulukosta Java 1.5:n geneerisyytt
 * ja reflektiona kytten niin, ett voi tallentaa vain
 * alkiota jotka toteuttavat tietyn rajapinnan ja uusi
 * alkio luodaan lisyksen yhteydess.
 * @author Vesa Lappalainen
 * @version 1.0, 02.03.2002
 * @version 1.1, 01.03.2005
 */

interface LukuRajapinta {
  public void set(int i);
  public int get();
}

class Luku implements LukuRajapinta {
  int luku;
  public Luku() {}
  public Luku(int i) { luku = i; }
  public int get() { return luku; }
  public void set(int i) { luku = i; }
  public String toString() { return ""+luku; }
}

public class TaulukkoGenExt<TYPE extends LukuRajapinta> {

  public static class TaulukkoTaysiException extends Exception {
    TaulukkoTaysiException(String viesti) { super(viesti); }
  }

  private Class<TYPE> luokka; // = TYPE.class; Tyhm, nin ei voi sanoa...
  private TYPE alkiot[];
  private int lkm;

  public TaulukkoGenExt(Class<TYPE> luokka) {
    this(luokka,10);
  }

  public TaulukkoGenExt(Class<TYPE> luokka,int koko) {
    alkiot = (TYPE[])new Object[koko];
    this.luokka = luokka;
  }

  public void lisaa(int i) throws TaulukkoTaysiException {
    try {
      TYPE uusi = luokka.newInstance();
      uusi.set(i);
      if (lkm >= alkiot.length)
        throw new TaulukkoTaysiException("Tila loppu");
      alkiot[lkm++] = uusi;
    }
    catch (IllegalAccessException ex) {
      throw new TaulukkoTaysiException("Laiton viittaus");
    }
    catch (InstantiationException ex) {
      throw new TaulukkoTaysiException("Laiton uuden luonti");
    }
  }

  public String toString() {
    StringBuffer s = new StringBuffer("");
    for (int i=0; i<lkm; i++)
      s.append(" " + alkiot[i]);
    return s.toString();
  }

  public void set(int i, int luku) throws IndexOutOfBoundsException {
    if ( ( i < 0 ) || ( lkm <= i ) ) throw new IndexOutOfBoundsException("i = " + i);
    alkiot[i].set(luku);
  }

  public int get(int i) throws IndexOutOfBoundsException {
    if ( ( i < 0 ) || ( lkm <= i ) ) throw new IndexOutOfBoundsException("i = " + i);
    return alkiot[i].get();
  }

  public static void main(String[] args) {
    TaulukkoGenExt<Luku> luvut = new TaulukkoGenExt<Luku>(Luku.class);
    try {
      luvut.lisaa(0); luvut.lisaa(2);
      luvut.lisaa(99);  // Tekee oikeasti luvut.lisaa(new Integer(99));
    } catch ( TaulukkoTaysiException e ) {
      System.out.println("Virhe: " + e.getMessage());
    }
    System.out.println(luvut);
    luvut.set(1,4);
    System.out.println(luvut);
    int luku = luvut.get(2);  // oik: luku = (Integer)(luvut.get(2)).intValue();
    System.out.println("Paikassa 2 on " + luku);
    try {
      luvut.set(21, 4);
    }
    catch (IndexOutOfBoundsException e) {
      System.out.println("Virhe: " + e.getMessage());
    }
  }
}
