package huone;


/**
 * Huoneet taulukoksi
 * @author Vesa Lappalainen
 * @version 1.0, 2.02.2003
 */
public class Asunto3 {

  private static final int MAX_HUONEITA = 6;

  private Huone2 huoneet[] = new Huone2[MAX_HUONEITA];
  private int huoneita = 0;

  /**
   * Alustetaan asunto ilman huoneita
   */
  public Asunto3() {  }

  /**
   * Listn asuntoon yksi huone
   * @param huone
   * @example
   * <pre name="test">
   *   Asunto3 kaksio = new Asunto3();
   *   kaksio.ala() ~~~ 0.0;
   *   kaksio.lisaa(new Huone2("olohuone",3,4,2,0));
   *   kaksio.ala() ~~~ 12.0;
   *   kaksio.lisaa(new Huone2("makuuhuone",3,3,2,0));
   *   kaksio.ala() ~~~ 21.0;
   * </pre>
   */
  public void lisaa(Huone2 huone) {
    if ( huoneita >= MAX_HUONEITA ) return;
    huoneet[huoneita] = huone; huoneita++;
  }

  /**
   * @return asunnon kaikkien huoneiden pinta-ala
   */
  public double ala() {
    double s = 0;
    for (int i=0; i<huoneita; i++) s += huoneet[i].ala();
    return s;
  }

  /**
   * @return asunnon kaikkien huoneiden tilavuus
   * <pre name="test">
   *   Asunto3 kaksio = new Asunto3();
   *   kaksio.lisaa(new Huone2("olohuone",3,4,2,0));
   *   kaksio.lisaa(new Huone2("makuuhuone",3,3,2,0));
   *   kaksio.tilavuus() ~~~ 42.0;
   * </pre>
   */
  public double tilavuus() {
    double s = 0;
    for (int i=0; i<huoneita; i++) s += huoneet[i].tilavuus();
    return s;
  }

  /**
   * @return asunnon kaikkien kaappien mr
   */
  public int getKaappeja() {
    int s = 0;
    for (int i=0; i<huoneita; i++) s += huoneet[i].getKaappeja();
    return s;
  }

  /**
   * Listn kaappi siihen huoneeseen jossa vhiten kaappeja
   * <pre name="test">
   *   Asunto3 kaksio = new Asunto3();
   *   Huone2 olo = new Huone2("olohuone",3,4,2,0);
   *   Huone2 makuu = new Huone2("makuuhuone",3,3,2,0);
   *   kaksio.lisaa(olo);
   *   kaksio.lisaa(makuu);
   *   kaksio.getKaappeja() === 0;
   *   kaksio.lisaaKaappeja();
   *   kaksio.getKaappeja() === 1;
   *   olo.getKaappeja()    === 1;
   *   makuu.getKaappeja()  === 0;
   *   kaksio.lisaaKaappeja();
   *   kaksio.getKaappeja() === 2;
   *   olo.getKaappeja()    === 1;
   *   makuu.getKaappeja()  === 1;
   *   kaksio.lisaaKaappeja();
   *   kaksio.getKaappeja() === 3;
   *   olo.getKaappeja()    === 2;
   *   makuu.getKaappeja()  === 1;
   * </pre>
   */
  public void lisaaKaappeja() {
    if ( huoneita == 0 ) return;
    Huone2 huone = huoneet[0];
    for (int i=1; i<huoneita; i++)
      if ( huoneet[i].getKaappeja() < huone.getKaappeja() )
        huone = huoneet[i];
    huone.lisaaKaappeja();
  }

  /**
   * @return asunnon tiedot ottamalla jokaisen huoneen tiedot ja yhteens kaikki summat
   * @example
   * <pre name="test">
   *   String tulos =  "Asunto:\n"+
   *                   "olohuone   12 m2 24 m3  0 kaappia\n"+
   *                   "makuuhuone  9 m2 18 m3  0 kaappia\n"+
   *                   "yhteens   21 m2 42 m3  0 kaappia";
   *   Asunto3 kaksio = new Asunto3();
   *   kaksio.lisaa(new Huone2("olohuone",3,4,2,0));
   *   kaksio.lisaa(new Huone2("makuuhuone",3,3,2,0));
   *   kaksio.toString() === tulos;
   * </pre>
   */
  public String toString() {
    StringBuilder s = new StringBuilder("Asunto:\n");
    for (int i=0; i<huoneita; i++) s.append(huoneet[i]+"\n");
    s.append(Huone2.muotoile("yhteens",ala(),tilavuus(),getKaappeja()));
    return s.toString();
  }

  /**
   * Kokeillaan Asunto-luokan toimintaa
   * @param args ei kytss
   */
  public static void main(String[] args)  {
    Asunto3 kaksio = new Asunto3();
    kaksio.lisaa(new Huone2("olohuone",3,4,2,0));
    kaksio.lisaa(new Huone2("makuuhuone",3,3,2,0));
    kaksio.lisaa(new Huone2("keitti",2,3,2,0));
    System.out.println(kaksio+"\n");
    for (int i=0; i<5; i++)
      kaksio.lisaaKaappeja();
    System.out.println(kaksio+"\n");
  }
}
