package valinta;

import fi.jyu.mit.ohj2.*;
/**
 * Luokka nestyksen vaihtoehdoille
 * @author Vesa Lappalainen
 * @version 1.0, 27.02.2003
 */
public class Vaihtoehdot {
  private Valinta alkiot[];
  private int lkm = 0;

  /**
   * Alustetaan vaihtoehtojen lukumr
   * @param maxKoko maksimimr vaihtoehtoja
   */
  public Vaihtoehdot(int maxKoko) {
    alkiot = new Valinta[maxKoko];
  }

  /**
   * Kysytn K/e kysymys Jatketaanko
   * @return true jos kyttj painaa K tai k
   */
  public static boolean jatketaan() {
    char c;
    System.out.println("Jatketaanko (K/e):");
    c = IO.odota_nappain("KE",'K',IO.MERKKI_ISOKSI);
    System.out.println(""+c);
    return ( c == 'K' );
  }

  
  /**
   * Lis uusi valinta.
   * @param uusi listtv valinta
   */
  public void lisaa(Valinta uusi) {
    if ( lkm >= alkiot.length ) return; // Jotakin hienompaa
    alkiot[lkm] = uusi;
    lkm++;
  }


  /**
   * Tulostetaan nestystilanne
   */
  private void tulostaAanet()  {
    System.out.println("Tilanne: ");
    for (int i=0; i<lkm; i++)
      System.out.println(Mjonot.fmt(alkiot[i].getTeksti(),-10) + ": " + alkiot[i].getMaara() + " ");
  }

  // #PACKAGEIMPORT
  /**
   * Etsitn valittua kohdetta kirjaimen perusteella
   * @param painettu etsittvn kohteen kirjain
   * @return lydetty kohde tai null jos ei lydy
   * 
   * @example
   * <pre name="test">
   * Vaihtoehdot vaihtoehdot = new Vaihtoehdot(3);
   * vaihtoehdot.lisaa(new Valinta('K',"Kyll"));
   * vaihtoehdot.lisaa(new Valinta('E',"Ei"));
   * vaihtoehdot.lisaa(new Valinta('H',"Ehk"));
   * vaihtoehdot.etsi('K').getTeksti() === "Kyll";
   * vaihtoehdot.etsi('E').getTeksti() === "Ei";
   * vaihtoehdot.etsi('H').getTeksti() === "Ehk";
   * vaihtoehdot.etsi('k') === null;
   * </pre>
   */
  public Valinta etsi(char painettu) {
    for (int i=0; i<lkm; i++ )
      if ( alkiot[i].getKirjain() == painettu )
         return alkiot[i];
    return null;
  }

  
  /**
   * Tulostetaan nestyskohteet ja odotetaan kyttjn painallusta
   * @return kyttjn painama nestyskohde
   */
  private char tulostaMenuJaOdota() {
    StringBuilder sallitut = new StringBuilder();
    char oletus = alkiot[0].getKirjain();

    for (int i=0; i<lkm; i++ ) {
      String tulostus = alkiot[i].getTulostus();
      System.out.println("  " + tulostus);
      sallitut.append(alkiot[i].getKirjain());
    }

    System.out.print("  :");
    return IO.odota_nappain(sallitut.toString(),oletus,IO.MERKKI_ISOKSI);
  }


  /**
   * nestyssilmukka, josta poistutaan kun kyttj ei halua en jatkaa
   * @param alkuteksti Ennen nestyst tulostettava teksti
   * @param toteamus kunkin nestyksen jlkeen tulostettava teksti
   */
  public void aanesta(String alkuteksti, String toteamus) {
    do {
      System.out.println(alkuteksti);
      System.out.println("");
      System.out.println("Valitse:");

      char painettu = tulostaMenuJaOdota();

      Valinta val = etsi(painettu);
      if ( val == null ) return; // thn jotakin fiksumpaa

      System.out.println("");
      System.out.println(toteamus + " " + val.getTeksti());

      val.lisaa();

      tulostaAanet();
    } while ( jatketaan() );

  }

}
