package valinta;

/**
 * Luokka nestyksen yhdelle vaihtoehdolle
 * @author Vesa Lappalainen
 * @version 1.0, 27.02.2003
 */
public class Valinta {

  private final char kirjain;
  private final String teksti;
  private int maara;

  
  /**
   * Alustetaan nestyskohde valituilla arvoilla
   * @param kirjain kirjain jolla nestetn
   * @param teksti mit nytetn nestyskohteeksi
   */
  public Valinta(char kirjain, String teksti) {
    this.kirjain = kirjain;
    this.teksti = teksti;
  }

  
  /**
   * Kohteen tulostettava teksti
   * @return esim. K = Kyll
   *  Valinta val = new Valinta('K',"Kyll"); 
   *  val.getTulostus() === "K = Kyll"; 
   */
  public String getTulostus() {
    return kirjain + " = " + teksti;
  }

  
  /**
   * Montako nt kohde on saanut
   * @return nien mr
   */
  public int getMaara() { return maara; }

  
  /**
   * Lis yksi ni kohteelle
   * @example
   * <pre name="test">
   *  Valinta val = new Valinta('K',"Kyll");
   *  val.getMaara() === 0;
   *  val.lisaa();
   *  val.getMaara() === 1;
   *  val.lisaa();
   *  val.getMaara() === 2;
   * </pre>
   */
  public void lisaa() { maara++; }

  
  /**
   * Kohteen nestyskirjain
   * @return kohteen kirjain
   */
  public char getKirjain() { return kirjain; }
  
  
  /**
   * Kohteen kuvaileva teksti
   * @return kohteen teksti
   */
  public String getTeksti() { return teksti; }


  /**
   * Testataan nestyskohdetta
   * @param args Ei kytss
   */
  public static void main(String[] args)  {
    Valinta val = new Valinta('K',"Kyll");
    String tulostus = val.getTulostus();
    System.out.println(tulostus);
    int lkm = val.getMaara();
    System.out.println(lkm);
    val.lisaa();
    lkm = val.getMaara();
    System.out.println(lkm);
  }
}

