package ohjausrak.polynomi.v2;

/**
 * Luokka toisen asteen polynomille ja sen nollakohdille
 * @author Vesa Lappalainen @version 1.0, 16.02.2003
 * @author Santtu Viitanen @version 1.1 12.7.2011
 * @example
 * <pre name="test">
 * new Polynomi2(1,2,1).juuret() === "x1 = -1.0 => P(x1) = 0.0  ja  x2 = -1.0 => P(x2) = 0.0"
 * new Polynomi2(2,1,0).juuret() === "x1 = -0.5 => P(x1) = 0.0  ja  x2 = 0.0 => P(x2) = 0.0"
 * new Polynomi2(1,-2,1).juuret() === "x1 = 1.0 => P(x1) = 0.0  ja  x2 = 1.0 => P(x2) = 0.0"
 * new Polynomi2(2,-1,0).juuret() === "x1 = 0.0 => P(x1) = 0.0  ja  x2 = 0.5 => P(x2) = 0.0"
 * new Polynomi2(2,1,1).juuret() === "Ei yhtn reaalijuuria"
 * new Polynomi2(2,0,0).juuret() === "x1 = -0.0 => P(x1) = 0.0  ja  x2 = 0.0 => P(x2) = 0.0"
 * new Polynomi2(0,2,1).juuret() === "x1 = -0.5 => P(x1) = 0.0  ja  x2 = -0.5 => P(x2) = 0.0"
 * new Polynomi2(0,0,1).juuret() === "Ei yhtn reaalijuuria"
 * </pre>
 */
@SuppressWarnings("all")
public class Polynomi2 {

  private double a,b,c,x1,x2;
  private int reaalijuuria;

  public Polynomi2(double a, double b, double c) {
    this.a = a; this.b = b; this.c = c;
    reaalijuuria = ratkaise2AsteenYhtalo();
  }

  private int ratkaise2AsteenYhtalo() {
    double D,SD;
    x1 = x2 = 0;
    if ( a == 0 )
      if ( b == 0 ) {
        if ( c == 0 ) return 1;
        else return 0;
      }
      else {
        x1 = x2 = -c/b;
        return 1;
      }
    else {
      D = b*b - 4*a*c;
      if ( D >= 0 ) {
        SD  = Math.sqrt(D);
        x1 = (-b-SD)/(2*a);
        x2 = (-b+SD)/(2*a);
        return 2;
      }
      else return -1;
    }
  }

  public static double P2(double x, double a, double b, double c) {
    return (a*x*x + b*x + c);
  }

  public double f(double x) { return P2(x,a,b,c); }
  public double getX1() { return x1; }
  public double getX2() { return x2; }
  public int getReaalijuuria() { return reaalijuuria; }

  @Override
public String toString() { return a + "x^2 + " + b + "x + " + c; }
  public String juuret() { 
      if ( getReaalijuuria() <= 0 ) return "Ei yhtn reaalijuuria";
      return
      "x1 = " + getX1() + " => P(x1) = " + f(getX1()) + "  ja  "+
      "x2 = " + getX2() + " => P(x2) = " + f(getX2());
  }

}
