package oliot.aika.metodi;

/**
 * Sisinen tilanvalvonta aikaan.
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika();
 *  Aika a2 = new Aika(13);
 *  Aika a3 = new Aika(14,25);
 *  a1.toString() === "00:00";  a2.toString() === "13:00";  
 *  a3.toString() === "14:25";
 *  a1.aseta(12,15); a2.aseta(16,-15);
 *  a1.toString() === "12:15"; a2.toString() === "15:45";
 *  a2.aseta(-49,-125); a2.toString() === "20:55";
 * </pre>
 */
public class Aika {

    private int h = 0, m = 0;


    /**
     * Alustaa ajan muotoon 00:00
     * @example
     * <pre name="test">
     * new Aika().toString() === "00:00";
     * </pre>
     */
    public Aika() {
        h = 0;
        m = 0;
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param h asetettavat tunnit
     * @example
     * <pre name="test">
     * new Aika(25).toString() === "01:00";
     * </pre>
     */
    public Aika(int h) {
        aseta(h, 0);
    }


    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     */
    public Aika(int h, int m) { // Muodostaja
        aseta(h, m);
    }


    /**
     * Lis aikaan valitun minuuttimrn
     * @param lisaMin listtv minuuttimr
     * @return montako vuorokautta ji yli
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27) === 0;    a1.toString() === "13:43";
     *  a1.lisaa(39) === 0;    a1.toString() === "14:22";
     * </pre>
     */
    public int lisaa(int lisaMin) {
        return aseta(getH(), getM() + lisaMin);
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param ih asetettava tuntimr
     * @param im asetettava minuuttimr
     * @return montako vuorokautta ji yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15"; 
     *  a1.aseta(15,45); a1.toString() === "15:45";
     *  a1.aseta(-49,-125); a1.toString() === "20:55";
     * </pre>
     */
    public final int aseta(int ih, int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit ville 0-59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit vlille 0-23

        if (tm < 0) {
            tm += 60;
            th--;
        }
        if (th < 0) {
            th += 24;
            vrk--;
        }

        this.h = th; // asetetaan lasketut arvot attribuutteeihin
        this.m = tm;
        return vrk; // motako vuorokautta ji yli
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param aika olio johon listn aikaa
     * @param ih asetettava tuntimr
     * @param im asetettava minuuttimr
     * @return montako vuorokautta ji yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  aseta(a1, 12,15); a1.toString() === "12:15"; 
     *  aseta(a1, 15,45); a1.toString() === "15:45";
     *  aseta(a1, -49,-125); a1.toString() === "20:55";
     * </pre>
     */
    public static int aseta(Aika aika, int ih, int im) {

        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit ville 0-59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit vlille 0-23

        if (tm < 0) {
            tm += 60;
            th--;
        } // negatiiviset arvot
        if (th < 0) {
            th += 24;
            vrk--;
        }

        aika.h = th; // asetetaan lasketut arvot attribuutteeihin
        aika.m = tm;
        return vrk; // motako vuorokautta ji yli
    }


    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", h, m);
    }


    /**
     * @return ajan tunnit
     */
    public int getH() {
        return h;
    }


    /**
     * @return ajan minuutit
     */
    public int getM() {
        return m;
    }

}
