package oliot.aika.sisesitys;

/**
 * Vaihdettu sisinen esitystapa
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 8.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);  a1.toString() === "13:10";
 *  a1.aseta(12,15); a1.toString() === "12:15";
 * </pre>
 */
public class Aika {

    private int yhtMin;

    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa. 
     * Ei osaa negatiivisia aikoja
     * @param h asetettavat tunnit
     * @param m asetettavat minuutit
     * @return yli- tai alijneet vuorokaudet
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15"; 
     *  a1.aseta(15,45); a1.toString() === "15:45";
     * </pre>
     */
    public final int aseta(int h, int m) {
        yhtMin = 60 * h + m;
        return yhtMin / 60 / 24; // vrk
    }

    /**
     * Alustetaan aika 0:00
     */
    public Aika() {
        aseta(0, 0);
    }

    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param h asetettavat tunnit
     */
    public Aika(int h) {
        aseta(h, 0);
    }

    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     */
    public Aika(int h, int m) {
        aseta(h, m);
    }

    /**
     * @return aika merkkijonona muodossa 12:05
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", getH(), getM());
    }

    /**
     * Lis aikaan halutun minuuttimrn
     * @param lisaMin listtv minuuttimr
     * @example
     * <pre name="test">
     * Aika aika = new Aika(11,12);
     * aika.lisaa(66); aika.toString() === "12:18";
     * </pre>
     */
    public void lisaa(int lisaMin) {
        aseta(0, yhtMin + lisaMin);
    }

    /**
     * Lis aikaan halutun minuuttimrn
     * @param aika olio johon minuutit listn
     * @param lisaMin listtv minuuttimr
     * Aika aika = new Aika(11,12);
     * aika.lisaa(aika, 66); aika.toString() === "12:18"
     */
    public static void lisaa(Aika aika, int lisaMin) {
        aika.lisaa(lisaMin);
    }

    /**
     * @return aika tunteina
     * @example
     * <pre name="test">
     * new Aika(11,12).getH() === 11;
     * </pre>
     */
    public int getH() {
        return yhtMin / 60;
    }

    /**
     * @return aika minuutteina
     * <pre name="test">
     * new Aika(11,12).getM() === 12;
     * </pre>
     */
    public int getM() {
        return yhtMin % 60;
    }

}
