package oliot.aika.valvonta;

/**
 * Sisinen tilanvalvonta Aikaan.
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);  a1.toString() === "13:10";
 *  a1.aseta(12,15); a1.toString() === "12:15";
 * </pre>
 */
public class Aika {

    private int h=0, m=0;

    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    public Aika(int h,int m) {
      this.h = h;
      this.m = m;
    }
    
    /**
     * Alustaa ajan muotoon 00:00
     * @example
     * <pre name="test">
     * new Aika().toString() === "00:00";
     * </pre>
     */
    public Aika() {
        h = 0; m = 0;
      }

    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param ih asetettava tuntimr
     * @param im asetettava minuuttimr
     * @return montako vuorokautta ji yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15"; 
     *  a1.aseta(15,45); a1.toString() === "15:45";
     *  a1.aseta(-49,-125); a1.toString() === "20:55";
     * </pre>
     */
    public final int aseta(int ih, int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit ville -59 - 59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit vlille 0-23
        
        if (tm<0) { tm += 60; th--; }  //negatiiviset arvot
        if (th<0) { th += 24; vrk--; }
        
        this.h = th; // asetetaan lasketut arvot attribuutteihin
        this.m = tm;
        return vrk; // motako vuorokautta ji yli
    }

    /**
     * Lis aikaan valitun minuuttimrn
     * @param lisaMin listtv minuuttimr
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27);    a1.toString() === "13:43";
     *  a1.lisaa(39);    a1.toString() === "14:22";
     * </pre>
     */
    public void lisaa(int lisaMin)  {
      int yhtMin = h * 60 + m + lisaMin;
      h = yhtMin / 60;
      m = yhtMin % 60;
    }
    
    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d",h, m);
    }
}
