package tiedosto;
import java.util.*;
import java.io.*;

/**
 * Lukujen lukeminen tiedostosta iteroituvaa Scanner-luokkaa kytten.
 * next palauttaa seuraavan rivin. 
 * Vikana se, ett ei voi iteroida kahta kertaa.
 * Java 1.7:n try:n avulla.
 * @author Vesa Lappalainen
 * @version 1.0, 05.03.2007
 */
@SuppressWarnings("javadoc")
public class TiedKaScannerI7 {

    public static class ScannerF implements Iterable<String>,Closeable {
        private Scanner scanner; 
        @SuppressWarnings("resource")
        public ScannerF(String filename) throws FileNotFoundException {
          scanner = new Scanner(new FileInputStream(new File(filename)));
        }
        @Override 
        public Iterator<String> iterator() { return new ScannerIterator(scanner); }
        @Override
        public void close() { scanner.close(); }
        
        public static class ScannerIterator implements Iterator<String> {
            Scanner scanner;
            public ScannerIterator(Scanner scanner) {
                this.scanner = scanner;
            }
            @Override
            public boolean hasNext() { return scanner.hasNextLine(); }
            @Override
            public String next() { return scanner.nextLine(); }
            @Override
            public void remove() { scanner.remove(); }
        }
    }
      
  
    /** 
     * Luetaan tiedosto ja lasketaan siell olevien lukujen
     * summa, keskiarvo ja mr.  Jos tiedoston nime ei
     * anneta, se on luvut.dat
     * @param args tiedoston nimi 
     * @example
     * <pre name="test">
     * #THROWS IOException
     * #import java.io.IOException;
     * #import fi.jyu.mit.ohj2.Suuntaaja;
     * #import fi.jyu.mit.ohj2.VertaaTiedosto;
     * String tiednimi = "tiedka7koe.txt";
     * VertaaTiedosto.kirjoitaTiedosto(tiednimi,"33\n11\nkissa\n5");
     * Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     * main(new String[]{tiednimi});
     * so.palauta();
     * String tulos = "Lukuja oli 3 kappaletta.\n"+
     *                "Niiden summa oli 49.00\n"+
     *                "ja keskiarvo oli 16.33\n";  
     * so.ero(tulos) === null;
     * VertaaTiedosto.tuhoaTiedosto(tiednimi);
     * Suuntaaja.StringOutput se = new Suuntaaja.StringOutput(true);
     * main(new String[]{"xxx.xxx"});
     * se.palauta();
     * se.ero("Tiedosto ei aukea! xxx.xxx (The system cannot find the file specified)\n") === null;
     * </pre>
     */
    public static void main(String[] args) {
        double summa = 0;
        int n = 0;
        String tiedNimi = "luvut.dat";
        if (args.length > 0 ) tiedNimi = args[0];
        
        try (ScannerF fi = new ScannerF(tiedNimi) ) {
            double luku;
            for (String s : fi) {
                try {
                    luku = Double.parseDouble(s);
                    summa += luku;
                    n++;
                } catch (NumberFormatException ex) {
                    // hyltn
                }
            }
        } catch (FileNotFoundException e) {
            System.err.println("Tiedosto ei aukea! " + e.getMessage());
            return;
        }

        double ka = 0;
        if (n > 0) ka = summa / n;
        System.out.printf("Lukuja oli %d kappaletta.%n",n);
        System.out.printf("Niiden summa oli %4.2f%n",summa);
        System.out.printf("ja keskiarvo oli %4.2f%n",ka);
    }
}
