<?xml version="1.0" encoding="ISO-8859-1"?>
<!--
     General string functions
     Miika Nurminen, june 2007
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

<xsl:variable name="lcletters">abcdefghijklmnopqrstuvwxyz</xsl:variable>
<xsl:variable name="ucletters">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
        
<xsl:template name="replace_group">
  <xsl:param name="content" />
  <xsl:param name="chars" select="/.." />
<!--  <xsl:value-of select="$chars/char[1]/@from" />-->
  <xsl:choose>
        <xsl:when test="count($chars)=0">
          <xsl:value-of select="$content" />
      </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="temp">
        <xsl:call-template name="replace-string">
          <xsl:with-param name="text" select="$content"/>
          <xsl:with-param name="from" select="$chars[1]/@from" />
          <xsl:with-param name="to" select="$chars[1]/@to"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:call-template name="replace_group">
          <xsl:with-param name="content" select="$temp" />
          <xsl:with-param name="chars" select="$chars[position() &gt; 1]" />
      </xsl:call-template>
    </xsl:otherwise>
</xsl:choose>  
</xsl:template>

<!-- TODO additional parametrization -->
<xsl:template name="replace_verbatim">
  <xsl:param name="content" />
  <xsl:param name="chars" select="/.." />
  <xsl:param name="mode" select="/.." /> <!-- jos moodi ei ole esimuotoiltu, kyttytyy kuten replace_group -->

  <xsl:choose>
     <xsl:when test="$mode='esimuotoiltu'">
          <xsl:choose>
                <xsl:when test="count($chars)=0">
                  <xsl:value-of select="$content" />
              </xsl:when>
            <xsl:otherwise>
              <xsl:variable name="temp">
                  <xsl:variable name="repto">
                      <xsl:choose>
                          <xsl:when test="$chars[1]/@verbatim"><xsl:value-of select="$chars[1]/@verbatim"/></xsl:when>
                          <xsl:otherwise><xsl:value-of select="$chars[1]/@to"/></xsl:otherwise>
                      </xsl:choose>
                  </xsl:variable>
                                <xsl:call-template name="replace-string">
                  <xsl:with-param name="text" select="$content"/>
                  <xsl:with-param name="from" select="$chars[1]/@from" />
                          <xsl:with-param name="to" select="$repto"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:call-template name="replace_verbatim">
                  <xsl:with-param name="content" select="$temp" />
                  <xsl:with-param name="chars" select="$chars[position() &gt; 1]" />
                  <xsl:with-param name="mode" select="$mode" />
              </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
     </xsl:when>
     <xsl:otherwise>
        <xsl:call-template name="replace_group">
            <xsl:with-param name="content" select="normalize-space($content)" />
            <xsl:with-param name="chars" select="$chars" />
        </xsl:call-template>
     </xsl:otherwise>
  </xsl:choose>
</xsl:template>

  <xsl:template name="append_if_nonempty">
    <xsl:param name="str"/>
    <xsl:param name="toappend"/>
    <xsl:choose>
      <xsl:when test="string-length($str)=0" />
      <xsl:otherwise>
        <xsl:value-of select="$str" /><xsl:value-of select="$toappend" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>                  

  
  <xsl:template name="starts_with_any">
    <xsl:param name="str"/>
    <xsl:param name="chars"/>
    <xsl:param name="default"/>
    <xsl:choose>
    <xsl:when test="string-length($chars) &gt; 0">
      <xsl:choose>
        <xsl:when test="starts-with($str,substring($chars,1,1))" />
        <xsl:otherwise>
  <xsl:call-template name="starts_with_any">
    <xsl:with-param name="str" select="$str"/>
    <xsl:with-param name="chars" select="substring($chars,2)" />
    <xsl:with-param name="default" select="$default" />
  </xsl:call-template></xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise><xsl:value-of select="$default" /></xsl:otherwise> <!-- doesn't start with any, return default -->
    </xsl:choose>
  </xsl:template>
  
  <!-- this should be done in sed instead... -->
  <xsl:template name="append_end_if_nonempty_nonalpha">
    <xsl:param name="str"/>
    <xsl:param name="toappend"/>
    <xsl:variable name="allowedchars"
select="'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'" />
    <xsl:choose>
      <xsl:when test="string-length($str)=0" />
      <xsl:otherwise><xsl:call-template name="starts_with_any">
        <xsl:with-param name="str" select="$str"/>
        <xsl:with-param name="chars" select="$allowedchars" />
        <xsl:with-param name="default" select="$toappend" />
  </xsl:call-template><xsl:value-of select="$str" /></xsl:otherwise>
  </xsl:choose>
  </xsl:template>                  
  
  <xsl:template name="append_end_if_nonempty">
    <xsl:param name="str"/>
    <xsl:param name="toappend"/>
    <xsl:choose>
      <xsl:when test="string-length($str)=0" />
      <xsl:otherwise>
        <xsl:value-of select="$toappend" /><xsl:value-of select="$str" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>                  
  
<xsl:template name="multiply_string">
  <xsl:param name="colstring" select="l" />
  <xsl:param name="colspan" select="1" />
  <xsl:choose>
    <xsl:when test="$colspan = 1"><xsl:value-of select="$colstring" /></xsl:when>
    <xsl:otherwise><xsl:value-of select="$colstring" />|<xsl:call-template name="multiply_string">
        <xsl:with-param name="colstring" select="$colstring" />
        <xsl:with-param name="colspan" select="$colspan - 1" />
      </xsl:call-template></xsl:otherwise>
  </xsl:choose>
</xsl:template>


 <!-- reusable replace-string function - taken from 
      http://aspn.activestate.com/ASPN/Cookbook/XSLT/Recipe/65426 -->
 <xsl:template name="replace-string">
    <xsl:param name="text"/>
    <xsl:param name="from"/>
    <xsl:param name="to"/>

    <xsl:choose>
      <xsl:when test="contains($text, $from)">

	<xsl:variable name="before" select="substring-before($text, $from)"/>
	<xsl:variable name="after" select="substring-after($text, $from)"/>
	<xsl:variable name="prefix" select="concat($before, $to)"/>

	<xsl:value-of select="$before"/>
	<xsl:value-of select="$to"/>
        <xsl:call-template name="replace-string">
	  <xsl:with-param name="text" select="$after"/>
	  <xsl:with-param name="from" select="$from"/>
	  <xsl:with-param name="to" select="$to"/>
	</xsl:call-template>
      </xsl:when> 
      <xsl:otherwise>
        <xsl:value-of select="$text"/>  
      </xsl:otherwise>
    </xsl:choose>            
 </xsl:template>

<!-- used at least by course_combiner and taulukot --> 
  <xsl:template name="get_default">
    <xsl:param name="current" />
      <xsl:choose>
        <xsl:when test="string-length($current)=0">1</xsl:when>
        <xsl:otherwise><xsl:value-of select="$current" /></xsl:otherwise>
      </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
