package fi.jyu.mit.gui;

import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JLabel;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Perusluokka valitsimille
 * @author vesal
 * @version 17.2.2013
 * @param <T> Ksiteltv tyyppi
 *
 */
public abstract class AbstractChooser<T> extends JPanel implements ActionListener, IStringListChooser<T> {
    
    
    private static final long serialVersionUID = 1L;

    private final JLabel labelValinta = new JLabel("Valitse");
    private String mnemonics = "";

    private final ArrayList<SelectionChangeListener<T>> selectionChangeListenerList = new ArrayList<SelectionChangeListener<T>>(); 
    private int selectedIndex = -1;
    private boolean setting = false; 
    private IntRing indices = new IntRing(1);
    private boolean preventAction = false;
    
    /**
     * @wbp.nonvisual location=72,351
     */
    private GenStringListModel<T> stringListModel = new GenStringListModel<T>();

    /**
     *  Alustetaan pohjaluokka
     */
    public AbstractChooser() {
        setLayout(new BorderLayout(0, 0));
        add(labelValinta, BorderLayout.NORTH);
        labelValinta.setLabelFor(this);
    }
    

    /**
     * Mihin otsikkoteksti asetettu
     * @return mihin asetettu
     */
    @Override
    public Direction getCaptionPosition() {
        if ( !labelValinta.isVisible() ) return Direction.HIDDEN;
        @SuppressWarnings("cast")
        Object c = ((BorderLayout)getLayout()).getConstraints(labelValinta);
        String s = (String)c;
        for ( Direction d : Direction.values() )
            if ( s.equalsIgnoreCase(d.toString()) ) return d;
        return Direction.HIDDEN;
    }

    
    /**
     * Mihin otsikkoteksti asetettu
     * @param dir uusi sijoitus
     */
    @Override
    public void setCaptionPosition(Direction dir) {
        if ( dir != Direction.HIDDEN ) { add(labelValinta,dir.toString()); return; }
        labelValinta.setVisible(false); 
    }
    
    
    /**
     * @see IStringListChooser#getSelectedChar()
     */
    @Override
    public char getSelectedChar() {
        char nappain = ' ';
        return nappain;
    }


    /**
     * @see IStringListChooser#getCaption()
     */
    @Override
    public String getCaption() {
        return labelValinta.getText();
    }

    
    /**
     * @return Teksti jolla nytetn Caption
     */
    public JLabel getCaptionLabel() {
        return labelValinta;
    }
    

    /**
     * @see IStringListChooser#setCaption(java.lang.String)
     */
    @Override
    public void setCaption(String text) {
        labelValinta.setText(text);
        labelValinta.setFont(getFont());
    }


    @Override
    public void setFont(Font font) {
        super.setFont(font);
        if ( labelValinta != null ) labelValinta.setFont(font);
    }

    
    /**
     * @see IStringListChooser#getKohteet()
     */
    @Override
    public String[] getKohteet() {
        return stringListModel.getItems();
    }

    
    /**
     * @see IStringListChooser#getKohteet()
     */
    @Override
    public int count() {
        if ( stringListModel == null ) return 0;
        return stringListModel.count();
    }

    
    /**
     * @param i monesko kohde
     * @return i's kohdeteksti
     */
    @Override
    public String getKohde(int i) {
        if ( stringListModel == null ) return "";
        if ( i < 0 || stringListModel.size() <= i ) return "";
        return stringListModel.elementAt(i);
    }
    

    /**
     * @see IStringListChooser#clear()
     */
    @Override
    public void clear() {
        if ( count() > 1)          // valittu indeksi ei saa olla 0 kun ruvetaan uusia lismn, en tied kenen vika???
            setSelectedIndex(1);
        else if ( count() == 1) {
            add("kana");           // nyt on pakko list yksi ett saadaan tyhjksi
            setSelectedIndex(1);
        }
        else 
            setSelectedIndex(-1);
        stringListModel.removeAllElements();
        mnemonics = "";
        
    }


    /**
     * @see IStringListChooser#add(java.lang.String, char)
     */
    @Override
    public void add(String s, char mnemo) {
        add(s, mnemo,null);
    }


    /**
     * @see IStringListChooser#add(java.lang.String)
     */
    @Override
    public void add(String s) {
        add(s, (char)0,null);
    }


    @Override
    public void add(String s, char mnemo, T obj) {
        this.stringListModel.addItem(s,obj);
        mnemonics += mnemo;
    }


    @Override
    public void add(String s, T obj) {
        add(s, (char)0,obj);
    }


    @Override
    public void setObjects(T[] objects) {
        int i=0;
        for (T obj:objects)
            stringListModel.setObject(i++, obj);
    }


    @Override
    public void setObjects(Collection<T> objects) {
        int i=0;
        for (T obj:objects)
            stringListModel.setObject(i++, obj);
    }
    
    
    @Override
    public T[] getObjects() {
        return stringListModel.getObjects();
    }


    @Override
    public T getObject(int i) {
        return stringListModel.getObject(i);
    }


    @Override
    public ArrayList<T> getObjectList() {
        return stringListModel.getObjectList();
    }

    
    /**
     * @see IStringListChooser#setKohteet(java.lang.String[])
     */
    @Override
    public void setKohteet(String[] kohteet) {
        clear();
        GenStringListModel<T> model = getModel();
        //setModel(null);
        for (String s : kohteet)
            //add(s);
            model.addItem(s);
        setModel(model);
        setSelectedIndex(selectedIndex);
    }


    /**
     * @see IStringListChooser#getMnemonics()
     */
    @Override
    public String getMnemonics() {
        return mnemonics;
    }


    /**
     * @param i monesko kirjain palautetaan
     * @return i's pikavalintakirjain
     */
    @Override
    public String getMnemonics(int i) {
        if ( i < 0 ) return "";
        if ( i >= mnemonics.length() ) return "";
        return ""+mnemonics.charAt(i);
    }


    /**
     * @see IStringListChooser#setMnemonics(java.lang.String)
     */
    @Override
    public void setMnemonics(String mnemonics) {
        this.mnemonics = mnemonics;
    }


    /**
     * @see IStringListChooser#getSelectedText()
     */
    @Override
    public String getSelectedText() {
        int i = getSelectedIndex();
        if ( i >= 0 ) return stringListModel.get(i);
        return "";
    }


    @Override
    public T getSelectedObject() {
        int i = getSelectedIndex();
        if ( i >= 0 ) return getObject(i);
        return null;
    }


    /**
     * Palauttaa valittujen kohdilla olevat oliot
     * @param idxs indeksit, joiden mukaan oliot palautetaan
     * @return valitut oliot, tyhj lista jos ei liitetty tai mitn ei valittuna
     */
    @Override
    public T[] getObjects(int[] idxs) {
        @SuppressWarnings("unchecked")
        T[] result = (T[]) (new Object[idxs.length]);
        int i = 0;
        for (int idx : idxs) result[i++] = getObject(idx);
        return result;
    }
    
    
    /**
     * Palauttaa valittujen kohdilla olevat oliot
     * @return valitut oliot, tyhj lista jos ei liitetty tai mitn ei valittuna
     */
    @Override
    public T[] getSelectedObjects() {
        return getObjects(getSelectedIndices());
    }
    
    
    /**
     * Palauttaa valittujen kohdilla olevat oliot
     * @return valitut oliot, tyhj lista jos ei liitetty tai mitn ei valittuna
     */
    @Override
    public T[] getSelectedObjectsSelectionOrder() {
        return getObjects(getSelectedIndicesSelectionOrder());
    }
    
    
    /**
     * @param idxs indeksit, joiden mukaan valitut palautetaan
     * @return valitut tekstit
     */
    @Override
    public String[] getTexts(int[] idxs) {
        @SuppressWarnings("unchecked")
        String[] result = new String[idxs.length];
        int i = 0;
        for (int idx : idxs) result[i++] = getKohde(idx);
        return result;
    }

    
    /**
     * @return valitut tekstit
     */
    @Override
    public String[] getSelectedTexts() {
        return getTexts(getSelectedIndices());
    }

    
    /**
     * @return valitut tekstit
     */
    @Override
    public String[] getSelectedTextsSelectionOrder() {
        return getTexts(getSelectedIndicesSelectionOrder());
    }

    
    /**
     * @param idxs indeksit, joiden perusteella kirjaimet palautetaan 
     * @return valittujen kirjaimet
     */
    @Override
    public String getChars(int[] idxs) {
        @SuppressWarnings("unchecked")
        StringBuilder result = new StringBuilder();
        for (int idx : idxs) result.append(getMnemonics(idx));
        return result.toString();
    }

    
    /**
     * @return kaikki valittujen kirjaimet
     */
    @Override
    public String getSelectedChars() {
        return getChars(getSelectedIndices());
    }

    
    /**
     * @return kaikki valittujen kirjaimet valitussa jrjestyksess
     */
    @Override
    public String getSelectedCharsSelectionOrder() {
        return getChars(getSelectedIndicesSelectionOrder());
    }

    
    /**
     * @see IStringListChooser#actionPerformed(java.awt.event.ActionEvent)
     */
    @Override
    public void actionPerformed(ActionEvent e) {
       if ( isPreventAction() ) return;
       if ( setting ) return;
       if ( !setting) selectedIndex = getSelectedIndexIntern(); 
       for (SelectionChangeListener<T> listener:selectionChangeListenerList)
           listener.selectionChange(this);
    }

    
    /**
     * @see IStringListChooser#addSelectionChangeListener(SelectionChangeListener)
     */
    @Override
    public void addSelectionChangeListener(SelectionChangeListener<T> listener) {
        selectionChangeListenerList.add(listener);
    }
   
    
    
    /**
     * Asettaa mallin
     * @param model mallilista jota kytetn
     */
    public void setModel(GenStringListModel<T> model) {
        stringListModel = model;
    }
    
    
    /**
     * @return kytetty listamalli
     */
    public GenStringListModel<T> getModel() {
        return stringListModel;
    }

    
    /**
     * Poistaa GUI-elementit
     */
    protected void clearGUI() {
        // kirjoitetaan tarvittaessa uudelleen toteuttavassa luokassa
    }


    private int maxChoices = 1;
    

    /**
     * @return suurin mr mit voi valita.  -1 = rajaton
     */
    @Override
    public int getMaxChoices() {
        return maxChoices;
    }
    
    
    /**
     * @param n suurin mr mit voi valita.  -1 = rajaton
     */
    @Override
    public void setMaxChoices(int n) {
        // oletuksena ei voi muuttaa
    }
    

    /**
     * @param n suurin mr mit voi valita.  -1 = rajaton
     */
    protected void setMaxChoicesIntern(int n) {
        maxChoices = n;
        indices = new IntRing(n);    
    }

    
    /**
     * Valittu indeksi
     */
    @Override
    public int getSelectedIndex() {
        if ( getKohteet().length <= selectedIndex ) return -1;
        return selectedIndex; // getSelectedIndexIntern();
    }


    /**
     * @see IStringListChooser#setSelectedIndex(int)
     */
    @Override
    public void setSelectedIndex(int selectedIndex) {
        setSelectedIndex(selectedIndex,true);
    }

    
    /**
     * @see IStringListChooser#setSelectedIndex(int)
     */
    @Override
    public void setSelectedIndex(int selectedIndex, boolean value) {
        if ( setting ) return;
        setting = true;
        this.selectedIndex = selectedIndex;
        setChecked(selectedIndex, value);
        setSelectedIndexIntern(selectedIndex,selectedIndex >= 0);
        setting = false;
        actionPerformed(null);
    }

    
    /**
     * Asetetaan sisinen tila ja poistetaan tarvittaessa ylimrinen asetus
     * @param index mink valinnan tila vaihdetaan
     * @param value mik on uusi arvo
     */
    protected void setChecked(int index, boolean value) {
        if ( value ) {
            if ( index < 0 || stringListModel.size() <= index ) return;
            // if ( indices.indexOf(index) >= 0 ) return;
            int toRemove = indices.add1(index);
            if ( toRemove >= 0 ) setSelectedIndexIntern(toRemove, false);
        }
        else indices.remove(index);
    }
    
    
    /**
     * Asetetaan valitut indeksit
     * @param indices lista asetettavien indekseist
     */
    @Override
    public void setSelectedIndices(int[] indices) {
        if ( setting ) return;
        setting = true;
        if ( indices == null || indices.length == 0 ) selectedIndex = -1;
        else selectedIndex = indices[0];
        setSelectedIndicesIntern(indices);
        actionPerformed(null);
        setting = false;
        actionPerformed(null);
    }
    
    
    /**
     * Asetetaan valitut indeksit
     * @param indices lista asetettavien indekseist
     */
    protected void setSelectedIndicesIntern(int[] indices) {
         for (int idx : indices) {
             if ( 0 <= idx && idx < stringListModel.size() ) { 
                 setChecked(idx, true);
                 setSelectedIndexIntern(idx, true);
             }
         }
             
    }
    
    
    /**
     * @return valittujen indeksien taulukko, tyhj jos mitn ei valittuna.
     */
    @Override
    public int[] getSelectedIndices() {
        return getSelectedIndicesIntern();
    }
    
    
    /**
     * @return valittujen indeksien taulukko valintajrjestyksess, tyhj jos mitn ei valittuna.
     */
    @Override
    public int[] getSelectedIndicesSelectionOrder() {
        return indices.getItems();
    }
    
    
    /** Tyhj indeksilista palautettavaksi */
    protected final static int[] emptyIndecies = {};
    
    /**
     * @return valitut indeksit omaan kyttn
     */
    protected int[] getSelectedIndicesIntern() {
        int i = getSelectedIndex();
        if ( i < 0 ) return emptyIndecies;
        return new int[]{i};
    }
    
    
    /**
     * @return valittujen lukumr
     */
    @Override
    public int getSelectedCount() {
        if ( getSelectedIndex() < 0 ) return 0;
        if ( maxChoices < 2 ) return 1;
        return getSelectedIndices().length;
    }
    
    
    /**
     * @return valittu indeksi omaan kyttn
     */
    protected abstract int getSelectedIndexIntern();
    
    
    /**
     * @param selectedIndex asetetaan valittu omaan kyttn
     * @param value asetettava arvo
     */
    protected abstract void setSelectedIndexIntern(int selectedIndex, boolean value);

    
    @Override
    public void clearSelection() {
        indices.clear();        
    }

    
    @Override
    public char getLastSelectedChar() {
        int i = getLastSelectedIndex();
        if ( i < 0 ) return ' ';
        if ( mnemonics.length() <= i ) return ' ';
        return mnemonics.charAt(i);
    }


    @Override
    public int getLastSelectedIndex() {
        return indices.getLast();
    }


    @Override
    public String getLastSelectedText() {
        return getKohde(getLastSelectedIndex());
    }


    @Override
    public T getLastSelectedObject() {
        return getObject(getFirstSelectedIndex());
    }
    
    
    @Override
    public char getFirstSelectedChar() {
        int i = getFirstSelectedIndex();
        if ( i < 0 ) return ' ';
        if ( mnemonics.length() <= i ) return ' ';
        return mnemonics.charAt(i);
    }


    @Override
    public int getFirstSelectedIndex() {
        return indices.getFirst();
    }


    @Override
    public String getFirstSelectedText() {
        return getKohde(getFirstSelectedIndex());
    }


    @Override
    public T getFirstSelectedObject() {
        return getObject(getLastSelectedIndex());
    }

    
    /**
     * @param index mink paikan valintaa tutkitaan
     * @return onko valittu vai ei
     */
    @Override
    public boolean isSelected(int index) {
        int i = indices.indexOf(index);
        return i >= 0;
    }


    /**
     * @return Onko actionit estetty
     */
    public boolean isPreventAction() {
        return preventAction;
    }


    /**
     * @param preventAction estetnk actionit
     */
    public void setPreventAction(boolean preventAction) {
        this.preventAction = preventAction;
    }


    
}
