package fi.jyu.mit.gui;

import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import java.awt.event.KeyListener;
import javax.swing.BoxLayout;
import java.awt.Component;
import java.awt.Dimension;
import javax.swing.SwingConstants;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.Document;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

/**
 * Luokka otsikon ja edit-kentn yhdistmiseksi 
 * @author vesal
 *
 */
public class EditPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private final JLabel label = new JLabel("nimi");
	private final JTextField edit = new JTextField();
	private final JLabel fill1 = new JLabel(" ");
	private final JLabel fill2 = new JLabel(" ");
	private int index;

	/**
	 * @return vapaasti kytettv indeksi
	 */
	public int getIndex() {
		return index;
	}


	/**
	 * Asetetaan indeksi
	 * @param index asetettava indeksi.
	 */
	public void setIndex(int index) {
		this.index = index;
	}


	/**
	 * Luodaan paneeli, siihen label ja edit-kentt ja muutama tytekentt.
	 */
	public EditPanel() {
		label.setHorizontalAlignment(SwingConstants.TRAILING);
		label.setPreferredSize(new Dimension(100, 14));
		label.setAlignmentX(Component.RIGHT_ALIGNMENT);
		label.setLabelFor(edit);
		
		edit.addFocusListener(new FocusAdapter() {
		    @Override
		    public void focusGained(FocusEvent fe) {
		        getEdit().selectAll();
		    }
		});
		edit.setColumns(10);
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
		add(label);
		add(fill1);
		add(edit);
		add(fill2);
		
		addUndoRedo(edit);
	}

	
	/**
	 * Laitetaan Edit-kenttn Undo/Redo -ominaisuus
	 * @param edit kentt johon ominaisuu laitetaan.
	 */
	public static void addUndoRedo(JTextField edit) {
	    final UndoManager undo = new UndoManager();
	    Document doc = edit.getDocument();

	    // Listen for undo and redo events
	    doc.addUndoableEditListener(new UndoableEditListener() {
	        @Override
            public void undoableEditHappened(UndoableEditEvent evt) {
	            undo.addEdit(evt.getEdit());
	        }
	    });

	    // Create an undo action and add it to the text component
	    edit.getActionMap().put("Undo",
	        new AbstractAction("Undo") {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent evt) {
	                try {
	                    if (undo.canUndo()) {
	                        undo.undo();
	                    }
	                } catch (CannotUndoException e) {
	                	// ei tehd mitn
	                }
	            }
	       });

	    // Bind the undo action to ctl-Z
	    edit.getInputMap().put(KeyStroke.getKeyStroke("control Z"), "Undo");

	    // Create a redo action and add it to the text component
	    edit.getActionMap().put("Redo",
	        new AbstractAction("Redo") {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent evt) {
	                try {
	                    if (undo.canRedo()) {
	                        undo.redo();
	                    }
	                } catch (CannotRedoException e) {
	                	// ei tehd mitn
	                }
	            }
	        });

	    // Bind the redo action to ctl-Y
	    edit.getInputMap().put(KeyStroke.getKeyStroke("control Y"), "Redo");
    }


	/**
	 * @return edit-kentss oleva teksti
	 */
    public String getText() {
		return edit.getText();
	}
	
	
    /**
     * Asetetaan edit-kenttn teksti
     * @param t asetettava teksti.
     */
	public void setText(String t) {
		edit.setText(t);
	}
	
	
	/**
	 * @return labelissa oleva teksti
	 */
	public String getCaption() {
		return label.getText();
	}
	
	
	/**
	 * Asetetaan labeliin teksti
	 * @param caption asetettava teksti
	 */
	public void setCaption(String caption) {
		label.setText(caption);
	}
	
	
	/**
	 * Listn kuuntelija edit-kenttn. 
	 */
	@Override
	public synchronized void addKeyListener(KeyListener l) {
		edit.addKeyListener(l);
	}
	
	
	/**
	 * @return edit-kentn viite
	 */
	public JTextField getEdit() {
		return edit;
	}
	
	
	/**
	 * @return labelin viite
	 */
	public JLabel getLabel() {
		return label;
	}
	

	/**
	 * Asetetaan edit-kentn leveys merkkein 
	 * @param columns montako saraketta 
	 */
	public void setColumns(int columns) {
		edit.setColumns(columns);
	}
	
	
	/**
	 * @return edit kentn leveys sarakkeina
	 */
	public int getColumns() {
		return edit.getColumns();
	}
		
	
	/**
	 * Asetetaan labelin leveys
	 * @param w asetettava leveys
	 */
	public void setLabelWidth(int w) {
		int h = getPreferredSize().height;
	    label.setPreferredSize(new Dimension(w, h));	
        label.setMinimumSize(new Dimension(w, h));    
        label.setMaximumSize(new Dimension(w, h));    
	}
	
}
