package fi.jyu.mit.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * Paneli, jossa on valittavat kohteet radiobuttoneina
 * @author vesal
 * @version 6.2.2011
 * @param <T> Ksiteltv tyyppi
 *
 */
public class GenCheckBoxChooser<T> extends AbstractChooser<T> implements
        ListDataListener {
    private static final long serialVersionUID = 1L;

    private final ArrayList<JCheckBox> buttons = new ArrayList<JCheckBox>();

    private final JPanel panelValinnat = new JPanel();
    private final JScrollPane scrollPane = new JScrollPane();
    private final JPanel panelKohteet = new JPanel();

    /**
     * Create the panel.
     */
    public GenCheckBoxChooser() {
        super();
        setMaxChoices(-1);
        setBorder(new CompoundBorder(
                new LineBorder(new Color(0, 0, 0), 1, true), new EmptyBorder(
                        10, 10, 10, 10)));
        panelValinnat.setBorder(new EmptyBorder(10, 10, 0, 10));

        add(panelValinnat, BorderLayout.CENTER);
        panelValinnat.setLayout(new BorderLayout(0, 0));
        scrollPane.setBorder(null);

        panelValinnat.add(scrollPane);
        panelKohteet.setBorder(null);

        panelKohteet.setLayout(new BoxLayout(panelKohteet, BoxLayout.Y_AXIS));
        scrollPane.setViewportView(panelKohteet);
        getModel().addListDataListener(this);

        // setModel(getModel());
    }

    /**
     * @see IStringListChooser#getSelectedChar()
     */
    @Override
    public char getSelectedChar() {
        String c = getMnemonics(getSelectedIndex());
        if (c.length() <= 0)
            return 0;
        return c.charAt(0);
    }

    /**
     * Poistaa GUI-elementit
     */
    @Override
    protected void clearGUI() {
        buttons.clear();
        panelKohteet.removeAll();
    }

    /**
     * @see IStringListChooser#add(java.lang.String, char)
     */
    @Override
    public void add(String s, char mnemo) {
        super.add(s, mnemo);
    }

    /**
     * Lis yhden Checkboxin
     * @param s listtvn teksti
     * @param mnemo kirjain jolla elementti valitaan
     * @return listty RadioButton
     */
    protected JCheckBox addGuiElement(String s, char mnemo) {
        JCheckBox cb = new JCheckBox(s);
        buttons.add(cb);
        panelKohteet.add(cb);
        cb.addActionListener(this);
        cb.setFont(getFont());
        if (mnemo != 0)
            cb.setMnemonic(mnemo);
        return cb;
    }

    /**
     * @see IStringListChooser#add(java.lang.String)
     */
    @Override
    public void add(String s) {
        add(s, (char) 0);
    }

    /**
     * @see IStringListChooser#setMnemonics(java.lang.String)
     */
    @Override
    public void setMnemonics(String mnemonics) {
        super.setMnemonics(mnemonics);
        for (int i = 0; i < buttons.size() && i < mnemonics.length(); i++) {
            JCheckBox cb = buttons.get(i);
            cb.setMnemonic(mnemonics.charAt(i));
        }
    }

    /**
     * @see IStringListChooser#getSelectedIndex()
     */
    @Override
    protected int getSelectedIndexIntern() {
        for (int i = 0; i < buttons.size(); i++) {
            if (buttons.get(i).isSelected())
                return i;
        }
        return -1;
    }

    /**
     * @see IStringListChooser#setSelectedIndex(int)
     */
    @Override
    protected void setSelectedIndexIntern(int selectedIndex, boolean value) {
        if (selectedIndex >= buttons.size())
            return;
        if (selectedIndex < 0)
            return;
        buttons.get(selectedIndex).setSelected(value);
    }

    
    /**
     * @return valittujen lukumr
     */
    @Override
    public int getSelectedCount() {
        int n = 0;
        for (JCheckBox cb : buttons)
            if (cb.isSelected())
                n++;
        return n;
    }

    /**
     * @return valitut indeksit omaan kyttn
     */
    @Override
    protected int[] getSelectedIndicesIntern() {
        int n = getSelectedCount();
        if (n == 0)
            return emptyIndecies;
        int[] idxs = new int[n];
        int i = 0;
        for (int idx = 0; idx < buttons.size(); idx++) {
            if (buttons.get(idx).isSelected())
                idxs[i++] = idx;
        }
        return idxs;
    }

    @Override
    public void setFont(Font font) {
        super.setFont(font);
        if (buttons != null)
            for (JCheckBox rb : buttons)
                rb.setFont(font);
    }

    /**
     * Lis kaikki komponentit uudelleen
     */
    protected void addAllGuiElements() {
        clearGUI();
        GenStringListModel<T> model = getModel();
        if (model == null)
            return;
        int i = 0;
        char c;
        String mns = getMnemonics();
        for (String s : getModel()) {
            c = 0;
            if (i < mns.length())
                c = mns.charAt(i);
            JCheckBox cb = addGuiElement(s, c);
            cb.setMnemonic(c);
        }
        updateUI();
    }

    /**
     * Asettaa mallin
     * @param model mallilista jota kytetn
     */
    @Override
    public void setModel(GenStringListModel<T> model) {
        GenStringListModel<T> oldModel = getModel();
        if (oldModel != null)
            oldModel.removeListDataListener(this);
        clearGUI();
        super.setModel(model);
        addAllGuiElements();
        model.addListDataListener(this);
    }

    @Override
    public void intervalAdded(ListDataEvent e) {
        addAllGuiElements();
    }

    @Override
    public void intervalRemoved(ListDataEvent e) {
        addAllGuiElements();
    }

    @Override
    public void contentsChanged(ListDataEvent e) {
        addAllGuiElements();
    }

    @Override
    public void clearSelection() {
        super.clearSelection();
        for (JCheckBox cb : buttons)
            cb.setSelected(false);

    }
    

    /**
     * @see IStringListChooser#actionPerformed(java.awt.event.ActionEvent)
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        if (buttons != null && e != null)
            for (int i = 0; i < buttons.size(); i++)
                if (buttons.get(i) == e.getSource()) {
                    setChecked(i, buttons.get(i).isSelected());
                    break;
                }
        super.actionPerformed(e);
    }

    
    /**
     * @param n suurin mr mit voi valita.  -1 = rajaton
     */
    @Override
    public void setMaxChoices(int n) {
        setMaxChoicesIntern(n);
    }
    

}
