package fi.jyu.mit.gui;

import java.awt.BorderLayout;

import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/**
 * Paneli, jossa on valittavat kohteet List-komponentissa
 * @author vesal
 * @version 20.3.2011
 * @param <T> Ksiteltv tyyppi
 *
 */
@SuppressWarnings({"rawtypes"})
public class GenListChooser<T> extends AbstractChooser<T> { 
    private static final long serialVersionUID = 1L;

    private final JScrollPane scrollPane = new JScrollPane();
    private final JList list = new JList();
    
    /**
     * Create the panel.
     */
    public GenListChooser() {
        super();

        setBorder(null);
        
        add(scrollPane, BorderLayout.CENTER);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setModel(getModel());
        
        list.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (  isPreventAction() ) return;
                actionPerformed(e);
            }
        });
        
        scrollPane.setViewportView(list);
        setKohteet(getKohteet());
    }


    /**
     * @return sisisen listan viite
     */
    public JList getList() {
        return list;
    }
    
    
    /**
     * @see IStringListChooser#getSelectedIndex()
     */
    @Override
    protected int getSelectedIndexIntern() {
        return list.getSelectedIndex();
    }


    /**
     * @return valittu indeksi omaan kyttn
     */
    @Override
    protected int[] getSelectedIndicesIntern() {
        return list.getSelectedIndices();
    }
    
    
    /**
     * @param n suurin mr mit voi valita.  -1 = rajaton
     */
    @Override
    public void setMaxChoices(int n) {
        int nm = n;
        if ( nm == 0 ) nm = 1;
        setMaxChoicesIntern(nm);
        if ( nm > 1 ) list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        else list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    }
    
    
    /**
     * @see IStringListChooser#setSelectedIndex(int)
     */
    @Override
    protected void setSelectedIndexIntern(int selectedIndex, boolean value) {
        // if ( value ) list.setSelectedIndex(selectedIndex);
        if ( value ) list.addSelectionInterval(selectedIndex, selectedIndex);
        else list.removeSelectionInterval(selectedIndex, selectedIndex);
        list.ensureIndexIsVisible(selectedIndex);
    }
    
    
    /**
     * Asettaa mallin
     * @param model mallilista jota kytetn
     */
    @Override
    public void setModel(GenStringListModel<T> model) {
        super.setModel(model);
        list.setModel(getModel());
    }


    @Override
    public void clearSelection() {
        super.clearSelection();
        list.clearSelection(); 
    }

    
    private boolean performingAction = false;
    
    
    /**
     * @param e listan muutostapahtuma
     */
    protected void actionPerformed(ListSelectionEvent e) {
        if ( performingAction || isPreventAction() || e!= null && e.getValueIsAdjusting()) {  return; }
        performingAction = true;
        if ( e != null ) {
            int si = list.getLeadSelectionIndex();
            for (int i = e.getFirstIndex(); i <= e.getLastIndex(); i++)
                if ( isSelected(i) != list.isSelectedIndex(i) ) {
                    setChecked(i, list.isSelectedIndex(i));
                }
            if ( isSelected(si) ) list.addSelectionInterval(si, si); // muuten ylspin menness menee vrin
        }
        super.actionPerformed(null);
        performingAction = false;
    }


    
}
