package fi.jyu.mit.gui;

import java.util.ArrayList;
import javax.swing.BoxLayout;
import javax.swing.ButtonModel;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Font;
import javax.swing.border.CompoundBorder;
import javax.swing.border.LineBorder;
import java.awt.Color;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JScrollPane;

/**
 * Paneli, jossa on valittavat kohteet radiobuttoneina
 * @author vesal
 * @version 6.2.2011
 * @param <T> Ksiteltv tyyppi
 *
 */
public class GenRadioButtonChooser<T> extends AbstractChooser<T> implements ListDataListener  {
    private static final long serialVersionUID = 1L;

    
    private final ArrayList<JRadioButton> buttons = new ArrayList<JRadioButton>();

    private final JPanel panelValinnat = new JPanel();
    // private final JLabel labelValinta = new JLabel("Valitse");
    private final ButtonGroup groupValinta = new ButtonGroup();
    private final JScrollPane scrollPane = new JScrollPane();
    private final JPanel panelKohteet = new JPanel();

    /**
     * Create the panel.
     */
    public GenRadioButtonChooser() {
        super();
        setBorder(new CompoundBorder(new LineBorder(new Color(0, 0, 0), 1, true), new EmptyBorder(10, 10, 10, 10)));
        panelValinnat.setBorder(new EmptyBorder(10, 10, 0, 10));

        add(panelValinnat, BorderLayout.CENTER);
        panelValinnat.setLayout(new BorderLayout(0, 0));
        scrollPane.setBorder(null);
        
        panelValinnat.add(scrollPane);
        panelKohteet.setBorder(null);
        
        panelKohteet.setLayout(new BoxLayout(panelKohteet, BoxLayout.Y_AXIS));
        scrollPane.setViewportView(panelKohteet);
        getModel().addListDataListener(this);

      //  setModel(getModel());
    }


    /**
     * @see IStringListChooser#getSelectedChar()
     */
    @Override
    public char getSelectedChar() {
        ButtonModel b = groupValinta.getSelection();
        if ( b == null ) return 0;
        char nappain = (char)b.getMnemonic();
        return nappain;
    }


    /**
     * Poistaa GUI-elementit
     */
    @Override
    protected void clearGUI() {
        for (JRadioButton rb : buttons)
            groupValinta.remove(rb);
        buttons.clear();
        panelKohteet.removeAll();
    }


    /**
     * @see IStringListChooser#add(java.lang.String, char)
     */
    @Override
    public void add(String s, char mnemo) {
        super.add(s,mnemo);
    }


    /**
     * Lis yhden RadioButtonin
     * @param s listtvn teksti
     * @param mnemo kirjain jolla elementti valitaan
     * @return listty RadioButton
     */
    protected JRadioButton addGuiElement(String s, char mnemo) {
        JRadioButton rb = new JRadioButton(s);
        buttons.add(rb);
        groupValinta.add(rb);
        panelKohteet.add(rb);
        rb.addActionListener(this);
        rb.setFont(getFont());
        if ( mnemo != 0 ) rb.setMnemonic(mnemo);
        return rb;
    }


    /**
     * @see IStringListChooser#add(java.lang.String)
     */
    @Override
    public void add(String s) {
        add(s, (char)0);
    }



    /**
     * @see IStringListChooser#setMnemonics(java.lang.String)
     */
    @Override
    public void setMnemonics(String mnemonics) {
        super.setMnemonics(mnemonics);
        for (int i = 0; i < buttons.size() && i < mnemonics.length(); i++) {
            JRadioButton rb = buttons.get(i);
            rb.setMnemonic(mnemonics.charAt(i));
        }
    }


    /**
     * @see IStringListChooser#getSelectedIndex()
     */
    @Override
    protected int getSelectedIndexIntern() {
        ButtonModel b = groupValinta.getSelection();
        for (int i = 0; i < buttons.size(); i++) {
            if ( buttons.get(i).getModel() == b ) return i;
        }
        return -1;
    }


    /**
     * @see IStringListChooser#setSelectedIndex(int)
     */
    @Override
    protected void setSelectedIndexIntern(int selectedIndex, boolean value) {
        if ( selectedIndex >= buttons.size() ) return;
        if ( selectedIndex < 0 ) return;
        buttons.get(selectedIndex).setSelected(value);
    }


    @Override
    public void setFont(Font font) {
        super.setFont(font);
        if ( buttons != null ) for (JRadioButton rb:buttons) rb.setFont(font);
    }


    /**
     * Lis kaikki komponentit uudelleen
     */
    protected void addAllGuiElements() {
        clearGUI();
        GenStringListModel<T> model = getModel();
        if ( model == null ) return;
        int i = 0;
        char c;
        String mns = getMnemonics();
        for (String s:getModel()) {
            c = 0;
            if ( i < mns.length() ) c = mns.charAt(i);  
            JRadioButton rb = addGuiElement(s,c);
            rb.setMnemonic(c);
        }    
        updateUI();
    }
    
    
    
    /**
     * Asettaa mallin
     * @param model mallilista jota kytetn
     */
    @Override
    public void setModel(GenStringListModel<T> model) {
        GenStringListModel<T> oldModel = getModel();
        if ( oldModel != null ) oldModel.removeListDataListener(this); 
        clearGUI();
        super.setModel(model);
        addAllGuiElements();
        model.addListDataListener(this);
    }


    @Override
    public void intervalAdded(ListDataEvent e) {
        addAllGuiElements();
    }


    @Override
    public void intervalRemoved(ListDataEvent e) {
        addAllGuiElements();
    }


    @Override
    public void contentsChanged(ListDataEvent e) {
        addAllGuiElements();
    }


    @Override
    public void clearSelection() {
        // for (JRadioButton rb:buttons) rb.
        
    }


    
    
}
