package fi.jyu.mit.gui;

import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultListModel;

/**
 * Listamalli merkkijonolistoja varten.  Kuhunkin jonoon voi liitt
 * halutessaan olion.
 * @author vesal
 * @version 20.3.2011
 * @param <T> Ksiteltv tyyppi
 */
public class GenStringListModel<T> extends DefaultListModel<String> implements ComboBoxModel<String>, Iterable<String> {
//public class GenStringListModel<T> extends DefaultListModel implements ComboBoxModel, Iterable<String> { // TODO 1.6

    private static final long serialVersionUID = 1L;
    private Object selected = "";
    private ArrayList<T> objects = new ArrayList<T>();

    
    /**
     * Poistaa kaikki jonot ja niihin liitetyt oliot
     */
    @Override
    public void clear() {
        removeAllElements();
    }

    
    /**
     * Poistaa kaikki jonot ja niihin liitetyt oliot
     */
    @Override
    public void removeAllElements() {
        super.removeAllElements();
        objects.clear();
    }
    
    /**
     * Laitetaan valinnat 
     * @param items merkkijonot joita voidaan valita
     */
    public void setItems(String[] items) {
        super.removeAllElements();
        objects.clear();
        for (String s:items) {
            super.addElement(s);
            objects.add(null);
        }
    }
    
    
    /**
     * @return taulukon kaikki jonot
     */
    public String[] getItems() {
        String[] items = new String[this.size()];
        for (int i=0; i<size(); i++)
            items[i] = get(i);
        return items;
    }
    
    
    /**
     * @return taulukon kaikki oliot
     */
    @SuppressWarnings("unchecked")
    public T[] getObjects() {
        T[] items = (T[])(new Object[this.size()]);
        for (int i=0; i<size(); i++)
            items[i] = getObject(i);
        return items;
    }
    
    
    /**
     * @return alkioiden lukumr
     */
    public int count() {
        return this.size();
    }
    
    
    /**
     * Palauttaa listan jossa jonoihin liitetyt oliot.
     * Listaa EI SAA ksitell muuta kuin kyd sit lpi.  
     * @return taulukon kaikki oliot
     */
    public ArrayList<T> getObjectList() {
        return objects;
    }
    
    /**
     * @param cla luokka jonka tyyppisi olioita halutaan
     * @return tyyppi T olevia olioita
     */
    @SuppressWarnings("unused")
    public ArrayList<T> getObjectList(T cla) {
        return getObjectList();
    }

    
    @Override
    public String get(int i) {
        if ( i < 0 ) return "";
        if ( i >= size() ) return "";
        return super.get(i).toString();
    }

    
    /**
     * Palauttaa i:n liitettyn olion
     * @param i monesko olio palautetaan
     * @return i.s olio
     */
    public T getObject(int i) {
        if ( i < 0 || objects.size() <= i ) return null;
        return objects.get(i);
    }
    
    
    /**
     * @param i monesko olio halutaan
     * @param cla mink luokan olio halutaan
     * @return tyyppi T oleva olio
     */
    @SuppressWarnings("unused")
    public T getObject(int i,T cla) {
        if ( i < 0 || objects.size() <= i ) return null;
        return objects.get(i);
    }

    
    
    /**
     * Palauttaa i:n liitettyn olion
     * @param i monesko olio asetetaan
     * @param obj olio joka asetetaan i:n jonoon liittyen
     * @return i.s olio
     */
    public T setObject(int i,T obj) {
        if ( i < 0 || objects.size() <= i ) return obj;
        objects.set(i,obj);
        return obj;
    }
    
    
    /**
     * Lis uuden jonon listaan
     * @param item listtv jono
     */
    public void addItem(String item) {
        addItem(item,null);
    }

    
    /**
     * Lis uuden jonon listaan
     * @param item listtv jono
     */
    @Override // TODO 1.6
    public void addElement(String item) {
        addItem(item.toString());
    }

    
    /**
     * Lis uuden jonon listaan
     * @param item listtv jono
     * @param obj jonoon liitettv objekti
     */
    public void addItem(String item,T obj) {
        super.addElement(item);
        objects.add(obj);
    }

    
    @Override
    public void setSelectedItem(Object anItem) {
        selected = anItem;
        fireContentsChanged(this, -1, -1);
    }

    
    @Override
    public Object getSelectedItem() {
        return selected;
    }


    @Override
    public Iterator<String> iterator() {
        return new StringListModelIterator();
    }
    
    /**
     * Iteraattori listaan
     * @author vesal
     * @version 20.3.2011
     */
    public class StringListModelIterator implements Iterator<String>{

        int index = -1;
        
        @Override
        public boolean hasNext() {
            return index < size()-1;
        }

        @Override
        public String next() {
            return get(++index);
        }

        @Override
        public void remove() {
          //  
        }
        
    }
    
    
}
