package fi.jyu.mit.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Rajapinta merkkijonovalitsemelle.
 * @author vesal
 * @version 17.2.2011
 * @param <T> ksiteltv tyyppi
 *
 */
public interface IStringListChooser<T> {

    
    /**
     * @return suurin mr mit voi valita.  -1 = rajaton
     */
    public abstract int getMaxChoices();
    
    
    /**
     * @param n suurin mr mit voi valita.  -1 = rajaton
     */
    public abstract void setMaxChoices(int n);
    
    
    /**
     * @return kirjain joka aiheutti valinnan
     */
    public abstract char getSelectedChar();

    
    /**
     * @return kirjain joka viimeksi aiheutti valinnan
     */
    public abstract char getLastSelectedChar();

    
    /**
     * @return kirjain joka valittu ensimmisen
     */
    public abstract char getFirstSelectedChar();

    
    /**
     * @param idxs indeksit joiden perusteella palautetaan
     * @return valittujen kirjaimet
     */
    public abstract String getChars(int[] idxs);


    
    /**
     * @return kaikki valittujen kirjaimet
     */
    public abstract String getSelectedChars();


    /**
     * @return kaikki valittujen kirjaimet valitussa jrjestyksess
     */
    public abstract String getSelectedCharsSelectionOrder();


    /**
     * @return valittujen lukumr
     */
    public abstract int getSelectedCount();
    
    
    /**
     * @return valintalistan ylpuolella oleva teksti
     */
    public abstract String getCaption();


    /**
     * Asetataan valintalistan ylpuolella oleva teksti
     * @param text listan ylpuolella oleva teksti
     */
    public abstract void setCaption(String text);


    /**
     * Mihin otsikkoteksti asetettu
     * @return mihin asetettu
     */
    public Direction getCaptionPosition();

    
    /**
     * Mihin otsikkoteksti asetettu
     * @param dir uusi sijoitus
     */
    public void setCaptionPosition(Direction dir);
        
        
    
    /**
     * @return the kohteet
     */
    public abstract String[] getKohteet();


    /**
     * @param i monesko kohde
     * @return i's kohdeteksti
     */
    public abstract String getKohde(int i);


    /**
     * Tyhjent koko valintalistan
     */
    public abstract void clear();


    /**
     * Lis yhden valintakohteen
     * @param s listtv teksti
     * @param mnemo pikanppin
     */
    public abstract void add(String s, char mnemo);


    /**
     * Lis yhden valintakohteen
     * @param s listtv teksti
     * @param mnemo pikanppin
     * @param obj liitettv objekti
     */
    public abstract void add(String s, char mnemo, T obj);


    /**
     * Lis yhden valintakohteen
     * @param s listtv teksti
     * @param obj liitettv objekti
     */
    public abstract void add(String s, T obj);


    /**
     * Lis yhden valintakohteen ilman pikanppint
     * @param s listtv teksti
     */
    public abstract void add(String s);


    /**
     * Asetetaan valintakohteet
     * @param kohteet taulukko kohteista
     */
    public abstract void setKohteet(String[] kohteet);


    /**
     * Asetetaan valintakohteisiin liittyvt oliot
     * @param objects taulukko olioista, pit olla samankokoinen kuin kohteiden mr
     */
    public abstract void setObjects(T[] objects);


    /**
     * Asetetaan valintakohteisiin liittyvt oliot
     * @param objects taulukko olioista, pit olla samankokoinen kuin kohteiden mr
     */
    public abstract void setObjects(Collection<T> objects);

    
    /**
     * @return taulukko jonoihin liitetyist oliosta
     */
    public abstract T[] getObjects();
    

    /**
     * Palauttaa valittujen kohdilla olevat oliot
     * @param idxs indeksit, joiden mukaan oliot palautetaan
     * @return valitut oliot, tyhj lista jos ei liitetty tai mitn ei valittuna
     */
    public T[] getObjects(int[] idxs); 
        
        
    /**
     * Palauttaa i:teen jonoon liitetyn olion 
     * @param i monesko olio palautetaan
     * @return i:teen jonoon liitetty olio 
     */
    public abstract T getObject(int i);

    
    /**
     * @return taulukko jonoihin liitetyist oliosta
     */
    public abstract ArrayList<T> getObjectList();
    

    /**
     * @return pikavalintakirjaimet
     */
    public abstract String getMnemonics();


    /**
     * @param i monesko kirjain palautetaan
     * @return i's pikavalintakirjain
     */
    public abstract String getMnemonics(int i);


    /**
     * @param mnemonics asetettavat pikavalintakirjaimet
     */
    public abstract void setMnemonics(String mnemonics);


    /**
     * @return valittu indeksi
     */
    public abstract int getSelectedIndex();


    /**
     * @return viimeiseksi valittu
     */
    public abstract int getLastSelectedIndex();


    /**
     * @return ekana valittu
     */
    public abstract int getFirstSelectedIndex();


    /**
     * @param index mink paikan valintaa tutkitaan
     * @return onko valittu vai ei
     */
    public abstract boolean isSelected(int index);


    /**
     * @return valittujen indeksien taulukko, tyhj jos mitn ei valittuna.
     */
    public int[] getSelectedIndices();

    
    /**
     * @return valittujen indeksien taulukko valintajrjestyksess, tyhj jos mitn ei valittuna.
     */
    public int[] getSelectedIndicesSelectionOrder();

    /**
     * @param selectedIndex asettava indeksi
     */
    public abstract void setSelectedIndex(int selectedIndex);


    /**
     * @param selectedIndex asettetva indeksi
     * @param value valittu vai ei
     */
    public abstract void setSelectedIndex(int selectedIndex, boolean value);
    
    
    /**
     * @return valittu teksti
     */
    public abstract String getSelectedText();

    
    /**
     * @return viimeiseksi valittu teksti
     */
    public abstract String getLastSelectedText();

    
    /**
     * @return ekana valittu teksti
     */
    public abstract String getFirstSelectedText();

    
    /**
     * @param idxs indeksit, joiden mukaan valitut palautetaan
     * @return valitut tekstit
     */
    public String[] getTexts(int[] idxs);
        
        
    /**
     * @return valitut tekstit listan jrjestyksess
     */
    public abstract String[] getSelectedTexts();

    
    /**
     * @return valitut tekstit valintajrjestyksess
     */
    public abstract String[] getSelectedTextsSelectionOrder();

    
    /**
     * Asetetaan valitut indeksit
     * @param indices lista asetettavien indekseist
     */
    public void setSelectedIndices(int[] indices);

    
    
    /**
     * Palauttaa valitun jonon kohdalla olevan olion
     * @return valittu olio, null jos ei liitetty tai mitn ei valittuna
     */
    public T getSelectedObject();
    

    /**
     * Palauttaa valitun viimeiseksi valitun jonon kohdalla olevan olion
     * @return valittu olio, null jos ei liitetty tai mitn ei valittuna
     */
    public T getLastSelectedObject();
    

    /**
     * Palauttaa valitun ekana valitun jonon kohdalla olevan olion
     * @return valittu olio, null jos ei liitetty tai mitn ei valittuna
     */
    public T getFirstSelectedObject();
    

    /**
     * @return valitut oliot, tyhj lista jos ei liitetty tai mitn ei valittuna
     */
    public T[] getSelectedObjects();
    
    
    /**
     * @return valitut oliot valintajrjestyksess, tyhj lista jos ei liitetty tai mitn ei valittuna
     */
    public T[] getSelectedObjectsSelectionOrder();
    
    /**
     * Tapahtuma joka tulee kun alkio valitaan
     * @param e taphatuman tiedot
     */
    public abstract void actionPerformed(ActionEvent e);


    /**
     * Listn kuuntelija kuuntelemaan valinnan muutosta
     * @param listener uusi kuuntelija
     */
    public abstract void addSelectionChangeListener(SelectionChangeListener<T> listener);

    
    /**
     * Poistaa kaikki valinnat 
     */
    public abstract void clearSelection();


    /**
     * @return alkioiden lukumr
     */
    public abstract int count();

}