package fi.jyu.mit.gui;

import java.awt.Color;

import javax.swing.JScrollPane;
import javax.swing.JTable;

/**
 * Rajapinta "uusille" taulukkoluokille
 * @author vesal
 * @version 24.4.2011
 */
public interface IStringTable {

    /**
     * Tarkistaa kuuntelijoilta onko joku tyytymtn asetettavaan arvoon
     * @param row    modelin rivin johon yritetn asettaa
     * @param column modelin sarake johon yritetn asettaa
     * @param s jono jota yritetn asettaa
     * @return null jos ok, muuten virheilmoitus
     */
    public abstract String checkAttribute(int row, int column, Object s);


    /**
     * Vaihtaa pyydetyn solun taustavrin halutuksi. Taustavri ei muutu ennen kuin
     * toisin pyydetn.  Indeksi modelin indeksej.
     * @param row mill rivill
     * @param column miss sarakkeessa
     * @param color mik taustavri
     */
    public abstract void setCellColor(int row, int column, final Color color);


    /**
     * @return taulukon rivien mr
     */
    public abstract int getRowCount();


    /**
     * Etsii muotissa kysytyss paikassa olevan arvon ja palauttaa sen.
     * Koordinaatit mallin koordinaatteja
     * @param row milt rivilt etsitn
     * @param column mist sarakkeesta etsitn
     * @return alkion sisltm arvo
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    public abstract Object getValueAtModel(int row, int column);


    /**
     * Etsii muotissa kysytyss paikassa olevan arvon ja palauttaa sen.
     * Koordinaatit taulukon koordinaatteja.
     * @param row milt rivilt etsitn
     * @param column mist sarakkeesta etsitn
     * @return alkion sisltm arvo
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    public abstract Object getValueAt(int row, int column);


    /**
     * Etsii muotista valitussa paikassa olevan arvon.
     * @return valittu arvo
     * @throws IndexOutOfBoundsException solua ei valittu
     */
    public abstract Object getSelectedValue();


    /**
     * Kertoo mik olio on liimattu muotin kyseiseen riviin.
     * Koordinaatit mallin koordinaatteja
     * Palauttaa null jos riviin ei ole liimattu oliota tai rivi on huono.
     * @param row mihin riviin
     * @return riviin liimattu olio
     */
    public abstract Object getObjectAt(int row);

    /**
     * Palauttaa valitussa kohdassa olevan olion
     * @return riviin liimattu olio, null jos ei ole
     */
    public abstract Object getSelectedObject();


    /**
     * Taulukossa oleva muotti.
     * @return taulukon muotti
    public abstract DefaultTableModel getModel();
     */


    /**
     * Antaa taulukon kyttjn rikottavaksi.
     * @return taulukko
     */
    public abstract JTable getTable();

    
    /**
     * Muuttaa taulukon rivin indeksinmallin riviksi
     * @param row taulukon rivi
     * @return vastaava rivinumero mallin rivin
     */
    public int convertRowIndexToModel(int row);

    /**
     * Muuttaa taulukon sarakkeen indeksi mallin sarakkeeksi
     * @param col taulukon sarake
     * @return vastaava rivinumero mallin sarakkeena
     */
    public int convertColumnIndexToModel(int col);

    
    /**
     * @return valitun rivin numero
     */
    public int getSelectedRow();
    

    /**
     * @return valitun sarakkeen numero
     */
    public int getSelectedColumn();
 
    
    /**
     * @return valitun rivin numero modelissa
     */
    public int getSelectedRowAtModel();
    

    /**
     * @return valitun sarakkeen numero modelissa
     */
    public int getSelectedColumnAtModel();
    
    
    /**
     * Valistee taulukosta halutun solun
     * @param row solun rivi
     * @param col solun sarake
     */
    public void selectCell(int row, int col);
    
    
    /**
     * @return kytetty scrollpane jotta sit voidaan st
     */
    public JScrollPane getScrollPane();
    
}