package fi.jyu.mit.gui;

/**
 * Luotu 18.3.2011. Paranneltu toimintaa ja korjattu kloonaus 26.3.2011.
 * Matriisin jokaisella rivill on yht monta paikkaa kuin ensimmisell.
 * @author Samu Tuominen
 * @version 26.3.2011
 * @param <TYPE> tyyppi, jota matriisi tallentaa
 */
public class Matriisi<TYPE> {
    
    private TYPE[][] matriisi;
    private int riveja;
    /**
     * @return rivien mr
     */
    public int getRiveja() {
        return riveja;
    }
    private int sarakkeita;
    /**
     * @return sarakkeiden mr
     */
    public int getSarakkeita() {
        return sarakkeita;
    }
    
    
    /**
     * Muodostaa tyhjn matriisin, johon mahtuu aluksi 10 rivi ja saraketta.
     */
    public Matriisi() {
        this(10, 10);
    }
    
    
    /**
     * Muodostaa tyhjn matriisin johon mahtuu aluksi annetut mrt rivej
     * ja sarakkeita.<br/>
     * Tekee pienimmilln 10*10 matriisin pohjan.
     * @param riviMaara montako rivi halutaan mahdollista tilaa
     * @param sarakeMaara montako saraketta halutaan mahdollista tilaa
     */
    @SuppressWarnings("unchecked")
    public Matriisi(int riviMaara, int sarakeMaara) {
        matriisi = (TYPE[][]) new Object[Math.min(10,riviMaara)][Math.min(10,sarakeMaara)];
        riveja = 0;
        sarakkeita = 0;
    }
    
    
    /**
     * Tekee annetun nkisen matriisin. Kaikilla riveill on sama pituus kuin
     * ensimmisell.
     * @param alkudata sislt, joka matriisiin laitetaan
     */
    public Matriisi(TYPE[][] alkudata) {
        this(alkudata.length, alkudata[0].length);
        riveja = alkudata.length;
        sarakkeita = alkudata[0].length;
        
        for (int i=0; i < riveja; i++)
            for (int j=0; (j < sarakkeita && j < alkudata[i].length); j++)
                matriisi[i][j] = alkudata[i][j];
    }
    
    
    @Override
    public String toString() {
        StringBuilder sisalto = new StringBuilder();
        String vali;
        for (int i=0; i < riveja; i++) {
            vali = "";
            for (int j=0; j < sarakkeita; j++) {
                sisalto.append(vali);
                vali = " ";
                sisalto.append(matriisi[i][j]);
            }
            sisalto.append('\n');
        }
        return sisalto.toString();
    }
    
    
    /**
     * Antaa pyydetyss paikassa olevan arvon.
     * @param rivi milt rivilt etsitn
     * @param sarake mist sarakkeesta etsitn
     * @return muistissa oleva arvo
     * @throws IndexOutOfBoundsException huonot indeksit
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2"}});
     * matriisi.haeArvo(1, 2) === "6";
     * matriisi.haeArvo(3, 2) === null;
     * matriisi.asetaArvo("11", 3, 2);
     * matriisi.haeArvo(3, 2) === "11";
     * </pre>
     */
    public TYPE haeArvo(int rivi, int sarake) throws IndexOutOfBoundsException {
        if (rivi < 0 || rivi >= riveja)
            throw new IndexOutOfBoundsException("Ei ole tuollaista rivi!");
        if (sarake < 0 || sarake >= sarakkeita)
            throw new IndexOutOfBoundsException("Ei ole tuollaista saraketta!");
        return matriisi[rivi][sarake];
    }
    
    
    /**
     * Asettaa annetun arvon haluttuun alkioon.
     * @param arvo mik arvo alkioon asetetaan
     * @param rivi mille riville asetetaan
     * @param sarake mihin sarakkeeseen asetetaan
     * @return onnistuiko arvon asettaminen
     */
    public boolean asetaArvo(TYPE arvo, int rivi, int sarake) {
        if (rivi < 0 || rivi >= riveja)
            return false;
        if (sarake < 0 || sarake >= sarakkeita)
            return false;
        matriisi[rivi][sarake] = arvo;
        return true;
    }
    
// sarakkeiden ksittely
    
    /**
     * Lis annetun mrn tyhji sarakkeita matriisiin.
     * @param maara montako tyhj saraketta halutaan
     * @return onnistuiko sarakkeiden lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaSarakkeita(2);
     * matriisi.toString() === "1 2 3 null null\n4 5 6 null null\n7 8 9 null null\n1 2 3 null null\n";
     * </pre>
     */
    public boolean lisaaSarakkeita(int maara) {
        return lisaaSarakkeita(maara, sarakkeita);
    }
    
    
    /**
     * Lis annetun mrn tyhji sarakkeita matriisiin.
     * @param maara montako tyhj saraketta halutaan
     * @param minne monennenko sarakkeen paikalle laitetaan
     * @return onnistuiko sarakkeiden lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaSarakkeita(2, 2);
     * matriisi.toString() === "1 2 null null 3\n4 5 null null 6\n7 8 null null 9\n1 2 null null 3\n";
     * </pre>
     */
    @SuppressWarnings("unchecked")
    public boolean lisaaSarakkeita(int maara, int minne) {
        int paikka = minne;
        if (maara < 1 || paikka < 0) return false;
        if (paikka > sarakkeita) paikka = sarakkeita;
        
        int leveys = matriisi[0].length;
        while ( (sarakkeita + maara) > leveys ) {
            TYPE[][] matriisi2 = (TYPE[][]) new Object[matriisi.length][leveys * 2];
            
            for (int i=0; i < riveja; i++)
                for (int j=0; j < sarakkeita; j++)
                    matriisi2[i][j] = matriisi[i][j];
            
            matriisi = matriisi2;
            leveys = matriisi[0].length;
        }
        sarakkeita += maara;
        for (int i=0; i < riveja; i++) {
            for (int j=(sarakkeita - 1); j >= paikka; j--) {
                if (j < (paikka + maara)) {
                    matriisi[i][j] = null;
                    continue;
                }
                matriisi[i][j] = matriisi[i][j - maara];
            }
        }
        return true;
    }
    
    
    /**
     * Lis annetun sarakkeen matriisin rivien phn.
     * @param sarake listtv sarake
     * @return onnistuiko sarakkeen lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaSarake(new String[]{"kissa", "koira", "sarake", "pelto"});
     * matriisi.toString() === "1 2 3 kissa\n4 5 6 koira\n7 8 9 sarake\n1 2 3 pelto\n";
     * </pre>
     */
    public boolean lisaaSarake(TYPE[] sarake) {
        return lisaaSarake(sarake, sarakkeita);
    }
    
    

    /**
     * Lis annetun sarakkeen matriisiin halutulle sarakkeelle ja siirt loppuja
     * sarakkeita eteenpin.
     * @param sarake listtv sarake
     * @param minne monennenko sarakkeen paikalle laitetaan
     * @return onnistuiko sarakkeen lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaSarake(new String[]{"hevonen", "kana", "lehm", "torni"}, 0);
     * matriisi.toString() === "hevonen 1 2 3\nkana 4 5 6\nlehm 7 8 9\ntorni 1 2 3\n";
     * </pre>
     */
    @SuppressWarnings("unchecked")
    public boolean lisaaSarake(TYPE[] sarake, int minne) {
        int paikka = minne;
        if (sarake.length != riveja || paikka < 0) return false;
        if (paikka > sarakkeita) paikka = sarakkeita;
        
        int leveys = matriisi[0].length;
        if ( sarakkeita == leveys ) {
            TYPE[][] matriisi2 = (TYPE[][]) new Object[matriisi.length][leveys * 2];
            
            for (int i=0; i < riveja; i++)
                for (int j=0; j < sarakkeita; j++)
                    matriisi2[i][j] = matriisi[i][j];
            
            matriisi = matriisi2;
        }
        for (int i=0; i < riveja; i++)
            for (int j=sarakkeita; j > paikka; j--)
                matriisi[i][j] = matriisi[i][j - 1];
                
        for (int i=0; i < riveja; i++)
            matriisi[i][paikka] = sarake[i];
            
        sarakkeita++;
        return true;
    }
    
    
    /**
     * Poistaa pyydetyn sarakkeen.
     * @param i poistettavan sarakkeen indeksi
     * @return onnistuiko sarakkeen poistaminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.poistaSarake(0);
     * matriisi.toString() === "2 3\n5 6\n8 9\n2 3\n";
     * </pre>
     */
    public boolean poistaSarake(int i) {
        return poistaSarakkeita(i, i + 1);
    }
    
    
    /**
     * Poistaa pyydetyt sarakkeet.
     * @param alku monennestako sarakkeesta poistaminen aloitetaan
     * @param loppu monesko sarake on ensimminen, joka silytetn
     * @return onnistuiko sarakkeiden poistaminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.poistaSarakkeita(1, 5);
     * matriisi.toString() === "1\n4\n7\n1\n";
     * </pre>
     */
    public boolean poistaSarakkeita(int alku, int loppu) {
        int i2 = loppu;
        if (i2 > sarakkeita) i2 = sarakkeita;
        if (alku < 0 || alku >= i2) return false;
        
        for (int i=0; i < riveja; i++) {
            for (int j=alku; j < sarakkeita; j++) {
                if ((j - alku + i2) >= sarakkeita) {
                    matriisi[i][j] = null;
                    continue;
                }
                matriisi[i][j] = matriisi[i][j - alku + i2];
            }
        }
        sarakkeita = sarakkeita + alku - i2;
        return true;
    }
    
// rivien ksittely
    
    /**
     * Lis annetun mrn tyhji rivej matriisiin.
     * @param maara montako tyhj rivi halutaan
     * @return onnistuiko rivien lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaRiveja(2);
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\nnull null null\nnull null null\n";
     * </pre>
     */
    public boolean lisaaRiveja(int maara) {
        return lisaaRiveja(maara, riveja);
    }
    
    
    /**
     * Lis annetun mrn tyhji rivej matriisiin.
     * @param maara montako tyhj rivi halutaan
     * @param minne monennenko rivin paikalle laitetaan
     * @return onnistuiko rivien lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaRiveja(2, 2);
     * matriisi.toString() === "1 2 3\n4 5 6\nnull null null\nnull null null\n7 8 9\n1 2 3\n";
     * </pre>
     */
    @SuppressWarnings("unchecked")
    public boolean lisaaRiveja(int maara, int minne) {
        int paikka = minne;
        if (maara < 1 || paikka < 0) return false;
        if (paikka > riveja) paikka = riveja;
        
        int korkeus = matriisi.length;
        while ( (riveja + maara) > korkeus ) {
            TYPE[][] matriisi2 = (TYPE[][]) new Object[korkeus * 2][matriisi[0].length];
            
            for (int i=0; i < riveja; i++)
                for (int j=0; j < sarakkeita; j++)
                    matriisi2[i][j] = matriisi[i][j];
                    
            matriisi = matriisi2;
            korkeus = matriisi.length;
        }
        
        riveja += maara;
        for (int i=(riveja - 1); i >= paikka; i--) {
            for (int j=0; j < sarakkeita; j++) {
                if (i < (paikka + maara)) {
                    matriisi[i][j] = null;
                    continue;
                }
                matriisi[i][j] = matriisi[i - maara][j];
            }
        }
        return true;
    }
    
    
    /**
     * Lis annetun rivin matriisin sarakkeiden phn.
     * @param rivi listtv rivi
     * @return onnistuiko rivin lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaRivi(new String[]{"hevonen", "kana", "lehm"});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\nhevonen kana lehm\n";
     * </pre>
     */
    public boolean lisaaRivi(TYPE[] rivi) {
        return lisaaRivi(rivi, riveja);
    }
    
    
    /**
     * Lis annetun rivin matriisiin halutulle riville ja siirt loppuja rivej
     * eteenpin.
     * @param rivi listtv rivi
     * @param minne monennenko rivin paikalle laitetaan
     * @return onnistuiko rivin lisminen
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.lisaaRivi(new String[]{"hevonen", "kana", "lehm"}, 2);
     * matriisi.toString() === "1 2 3\n4 5 6\nhevonen kana lehm\n7 8 9\n1 2 3\n";
     * </pre>
     */
    @SuppressWarnings("unchecked")
    public boolean lisaaRivi(TYPE[] rivi, int minne) {
        int paikka = minne;
        if (rivi.length != sarakkeita || paikka < 0) return false;
        if (paikka > riveja) paikka = riveja;
        
        int korkeus = matriisi.length;
        if ( riveja == korkeus ) {
            TYPE[][] matriisi2 = (TYPE[][]) new Object[korkeus * 2][matriisi[0].length];
            
            for (int i=0; i < riveja; i++)
                for (int j=0; j < sarakkeita; j++)
                    matriisi2[i][j] = matriisi[i][j];
            
            matriisi = matriisi2;
        }
        
        for (int i=riveja; i > paikka; i--)
            for (int j=0; j < sarakkeita; j++)
                matriisi[i][j] = matriisi[i - 1][j];
                
        for (int j=0; j < sarakkeita; j++)
            matriisi[paikka][j] = rivi[j];
            
        riveja++;
        return true;
    }
    
    
    /**
     * Poistaa pyydetyn rivin.
     * @param i poistettavan rivin indeksi
     * @return onnistuiko rivin poisto
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.poistaRivi(0);
     * matriisi.toString() === "4 5 6\n7 8 9\n1 2 3\n";
     * </pre>
     */
    public boolean poistaRivi(int i) {
        return poistaRiveja(i, i + 1);
    }
    
    
    /**
     * Poistaa pyydetyt rivit.
     * @param alku monennestako rivist poistaminen aloitetaan
     * @param loppu monesko rivi on ensimminen, joka silytetn
     * @return onnistuiko rivien poisto
     * @example
     * <pre name="test">
     * Matriisi<String> matriisi = new Matriisi<String>(
     *          new String[][]{{"1", "2", "3"},
     *                         {"4", "5", "6"},
     *                         {"7", "8", "9", "10"},
     *                         {"1", "2", "3"}});
     * matriisi.toString() === "1 2 3\n4 5 6\n7 8 9\n1 2 3\n";
     * matriisi.poistaRiveja(1, 3);
     * matriisi.toString() === "1 2 3\n1 2 3\n";
     * </pre>
     */
    public boolean poistaRiveja(int alku, int loppu) {
        int i2 = loppu;
        if (i2 > riveja) i2 = riveja;
        if (alku < 0 || alku >= i2) return false;
        
        for (int i=alku; i < riveja; i++) {
            for (int j=0; j < sarakkeita; j++) {
                if ((i - alku + i2) >= riveja) {
                    matriisi[i][j] = null;
                    continue;
                }
                matriisi[i][j] = matriisi[i - alku + i2][j];
            }
        }
        riveja = riveja + alku - i2;
        return true;
    }
    
// muuta
    
    @SuppressWarnings("unchecked")
    @Override
    public Matriisi<TYPE> clone() {
        TYPE[][] sisalto = (TYPE[][]) new Object[riveja][sarakkeita];
        
        for (int i=0; i < riveja; i++)
            for (int j=0; j < sarakkeita; j++)
                sisalto[i][j] = matriisi[i][j];
        
        return new Matriisi<TYPE>(sisalto);
    }
    
}
