package fi.jyu.mit.gui;

import java.util.ArrayList;
import javax.swing.table.DefaultTableModel;

/**
 * Luotu 3.4.2011.
 * @author Samu Tuominen
 * @version 3.4.2011
 */
public class ObjectTable extends StringTable {

    private static final long serialVersionUID = 1L;


    /**
     * Luo uuden ObjectTable:n ja asettaa sisllksi annetut oliot. Sarakkeet nimetn
     * automaattisesti.<br/>
     * @param malli taulkkomalli, jota kytetn
     */
    public ObjectTable(final TableObject malli) {
        super(malli);
    }


    /**
     * Muodostaa uuden tyhjn ObjectTable:n.
     */
    public ObjectTable() {
        this(new DefaultTableModel());
    }


    /**
     * Muodostaa uuden ObjectTable:n annetulla sisltmodelilla.
     * @param model mit taulukkoon laitetaan sislle
     */
    public ObjectTable(DefaultTableModel model) {
        super(model);
    }




    /**
     * Muodostaa uuden ObjectTable:n ja alustaa sen annetuilla mrill tyhji rivej
     * ja sarakkeita.
     * @param riveja montako rivi halutaan
     * @param sarakkeita montako saraketta halutaan
     */
    public ObjectTable(int riveja, int sarakkeita) {
        this(new DefaultTableModel(riveja, sarakkeita));
    }


    /**
     * Muodostaa uuden ObjectTable:n ja alustaa sen annetuilla sisllll ja
     * sarakkeiden nimill.
     * @param sisalto mit taulukkoon aluksi laitetaan
     * @param nimet sarakkeiden nimet
     */
    public ObjectTable(Object[][] sisalto, Object[] nimet) {
        this(new DefaultTableModel(sisalto, nimet));
    }


    /**
     * Muodostaa uuden StringTable:n ja alustaa sen annetuilla sarakkeiden nimill ja
     * halutulla mrll tyhji rivej.
     * @param nimet sarakkeiden nimet
     * @param riveja montako rivi halutaan
     */
    public ObjectTable(Object[] nimet, int riveja) {
        this(new DefaultTableModel(nimet, riveja));
    }




    /**
     * Etsii muotissa kysytyss paikassa olevan arvon ja palauttaa sen.
     * Koordinaatit mallin koordinaatteja
     * @param row milt rivilt etsitn
     * @param column mist sarakkeesta etsitn
     * @return alkion sisltm arvo
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    @Override
    public Object getValueAtModel(int row, int column) {
        int mrow = table.convertRowIndexToModel(row);
        int mcolumn = table.convertColumnIndexToModel(column);
        Object value = table.getModel().getValueAt(mrow, mcolumn);
        if ( value == null ) return null;
        if ( value instanceof ObjectStringPair )
            return ((ObjectStringPair)value).getValue();
        return value;
    }


    /**
     * Asettaa annetun arvon haluttuun alkioon muottiin.
     * Koordinaatit mallin koordinaatteja
     * @param row mille riville asetetaan
     * @param column mihin sarakkeeseen asetetaan
     * @param value mik arvo alkioon asetetaan
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    @Override
    public void setValueAtModel(Object value, int row, int column) {
        Object obj = table.getModel().getValueAt(row, column);
        if ( obj == null ) {
            table.getModel().setValueAt(new ObjectStringPair(value, null),
                    row, column);
            return;
        }
        ObjectStringPair pari;
        if ( obj instanceof ObjectStringPair ) {
            pari = (ObjectStringPair)obj;
            pari.setValue(value);
        } else {
            pari = new ObjectStringPair(value, null);
            table.getModel().setValueAt(pari, row, column);
        }
    }


    /**
     * Asettaa annetun arvon haluttuun alkioon muottiin.
     * Koordinaatit mallin koordinaatteja
     * @param row mille riville asetetaan
     * @param column mihin sarakkeeseen asetetaan
     * @param value mik arvo alkioon asetetaan
     * @param refObj mihin olioon viitataan
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    public void setValueAtModel(Object value, int row, int column, Object refObj) {
        Object obj = table.getModel().getValueAt(row, column);
        if ( obj == null ) {
            table.getModel().setValueAt(new ObjectStringPair(value, refObj),
                    row, column);
            return;
        }
        ObjectStringPair pari;
        if ( obj instanceof ObjectStringPair ) {
            pari = (ObjectStringPair)obj;
            pari.setValue(value);
            pari.setObject(refObj);
        } else {
            pari = new ObjectStringPair(value, refObj);
            table.getModel().setValueAt(pari, row, column);
        }
    }


    /**
     * Haetaan soluun liimattu olio.
     * Koordinaatit mallin koordinaatteja
     * Palauttaa null vain jos soluun ei ole koskaan asetettu arvoa tai soluun on
     * erityisesti kiinnitetty null.
     * @param row mille riville
     * @param column mihin sarakkeeseen
     * @return soluun liimattu olio
     * @throws IndexOutOfBoundsException huono sijainti solulle
     */
    public Object getObjectAt(int row, int column) {
        Object obj = table.getModel().getValueAt(row, column);
        if ( obj == null ) {
            return null;
        }
        if ( obj instanceof ObjectStringPair ) {
            Object olio = ((ObjectStringPair)obj).getObject();
            if ( olio == null ) {
                return ((ObjectStringPair)obj).getValue();
            }
            return olio;
        }
        Object arvo = riviOliot.get(row);
        if ( arvo == null ) return obj;
        return arvo;
    }


    /**
     * Liimataan soluun kiinni olio.
     * Koordinaatit mallin koordinaatteja
     * @param obj kiinniliimattava olio
     * @param row mille riville
     * @param column mihin sarakkeeseen
     * @throws IndexOutOfBoundsException huono sijainti solulle
     */
    public void setObjectAt(Object obj, int row, int column) {
        Object value = table.getModel().getValueAt(row, column);
        if ( value == null ) {
            table.getModel().setValueAt(new ObjectStringPair(obj, obj), row, column);
        }
        ObjectStringPair pari;
        if ( value instanceof ObjectStringPair ) {
            pari = (ObjectStringPair)value;
            pari.setObject(obj);
        } else if ( value != null ) {
            pari = new ObjectStringPair(value.toString(), obj);
            table.getModel().setValueAt(pari, row, column);
        }
    }


    @Override
    protected DefaultTableModel getNewTableModel(final ArrayList<Class<?>> sarakeTyypit) {
        return new DefaultTableModel(
                new Object[0][sarakeTyypit.size()],
                getColumnNames()
                ) {
                    private static final long serialVersionUID = 1L;
                    Class<?>[] columnTypes = sarakeTyypit.toArray(new Class<?>[0]);


                    @Override
                    public Class<?> getColumnClass(int columnIndex) {
                        return columnTypes[columnIndex];
                    }


                    @Override
                    public Object getValueAt(int row, int column) {
                        Object value = super.getValueAt(row, column);
                        if ( value == null ) return null;
                        try {
                            return ((ObjectStringPair)value).getValue();
                        } catch (ClassCastException e) {
                            return value;
                        }
                    }
                };
    }
    
    
    /*
     * @param sisalto taulukon sislt
    public void setTableObject(final TableObject sisalto) {
        oliot = sisalto;
        // alustetaan sarakkeet
        final ArrayList<Class<?>> sarakeTyypit = new ArrayList<Class<?>>();
        for (int i = 0; i < oliot.getAttributeCount(); i++) {
            sarakeTyypit.add(oliot.getClass(i));
            otsikot.add(oliot.getHeader(i));
        }
        table.setModel(new DefaultTableModel(
                new Object[0][sarakeTyypit.size()],
                getColumnNames()
                ) {
                    private static final long serialVersionUID = 1L;
                    Class<?>[] columnTypes = sarakeTyypit.toArray(new Class<?>[0]);


                    @Override
                    public Class<?> getColumnClass(int columnIndex) {
                        return columnTypes[columnIndex];
                    }


                    @Override
                    public Object getValueAt(int row, int column) {
                        Object value = super.getValueAt(row, column);
                        if ( value == null ) return null;
                        try {
                            return ((ObjectStringPair)value).getValue();
                        } catch (ClassCastException e) {
                            return value;
                        }
                    }
                });
        setCellEditors();
        // alustetaan rivit
        for (int j = 0; j < oliot.getObjectCount(); j++) {
            addRow(j);
        }
    }
*/


}
