package fi.jyu.mit.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.DefaultCellEditor;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

/**
 *
 * @author Samu Tuominen
 * @version 4.4.2011
 */
public class StringCellEditor extends DefaultCellEditor {
    
    private static final long serialVersionUID = 1L;
    
    private JTextField editorField;
    private int nr;
    private int kentta;
    
    private int irow;
    /**
     * @return the irow
     */
    public int getIrow() {
        return irow;
    }


    private int icol;

    /**
     * @return the icol
     */
    public int getIcol() {
        return icol;
    }


    
    /** Viite editoitavaan olioon */
    protected final TableObject oliot;
    private String ok = null;
    /** Kohdalla oleva arvo */
    protected String currentValue;

    private int haling = SwingConstants.LEFT; 
    
    
    /**
     * @return miss paikassa editoidaan
     */
    public int getHaling() {
        return haling;
    }


    /**
     * @param haling miss paikassa editoidaan
     */
    public void setHaling(int haling) {
        this.haling = haling;
    }


    /**
     * Alustetaan editori tietty taulukkoa varten.
     * @param objects riveihin kiinni liimattavat oliot
     */
    public StringCellEditor(TableObject objects) {
        super(new JTextField());
        setClickCountToStart(2);
        oliot = objects;
        editorField = (JTextField)getComponent();
        editorField.setBorder(null);
        EditPanel.addUndoRedo(editorField);
    }
    
    
    /**
     * Asetetaan editorin fontti
     * @param font asetettava fontti
     */
    public void setFont(Font font) {
        editorField.setFont(font);
    }
    
    
    
    /**
     * @return editorin fontti
     */
    public Font getFont() {
        return editorField.getFont();
    }
    
    
    /**
     * @return mit kentt editori tll hetkell ksittelee
     */
    protected int getKentta() {
        return kentta;
    }
    
    
    /**
     * @return rivi jota editori tll hetkell ksittelee
     */
    protected int getNr() {
        return nr;
    }
    
    
    @Override
    public boolean stopCellEditing() {
       return ok == null;
    }
    
    
    /**
     * Palautetaan editori ja otetaan talteen mit solua editointi koskee.
     */
    @Override
    public Component getTableCellEditorComponent(final JTable table,
            Object value, boolean isSelected,
            final int row, final int column) {
        final JTextField cellEditor =
                (JTextField) super.getTableCellEditorComponent(table, value,
//                                                            true, row, column);
                                                              isSelected, row, column);
        cellEditor.setBackground(Color.white);
        nr = table.convertRowIndexToModel(row);
        kentta = table.convertColumnIndexToModel(column);
        irow = row;
        icol = column;
        KeyListener[] kuuntelijat = cellEditor.getKeyListeners();
        for (int i=0; i < kuuntelijat.length; i++)
            cellEditor.removeKeyListener(kuuntelijat[i]);
        if ( value == null ) currentValue = "";
        else currentValue = value.toString();
        cellEditor.setText(currentValue);
        setAlignment(cellEditor);
        cellEditor.selectAll();
//        ((ICTable)table).setError(getNr(), getKentta(), null);
        cellEditor.addKeyListener(new KeyAdapter() { 
            @SuppressWarnings("synthetic-access")
            @Override
            public void keyReleased(KeyEvent e) {
                if (e.getKeyChar() == '\n') {
                    table.clearSelection();
                    boolean enterNextColumn = true;
                    try {
                        if ( table instanceof StringTable.OTable ) {
                            enterNextColumn = ((StringTable.OTable)table).getOwner().isEnterGoesNextColumn();
                        }
                        int rd = 0;
                        int cd = 1;
                        if ( !enterNextColumn ) { rd = 1; cd = 0; }
                        table.addRowSelectionInterval(row + rd , row + rd );
                        table.addColumnSelectionInterval(column+cd, column+cd);
                    } catch (IllegalArgumentException ex) {
                        if ( !enterNextColumn ) {
                            table.addRowSelectionInterval(0, 0);
                            // table.addColumnSelectionInterval(column, column);
                            try {
                                table.addColumnSelectionInterval(column + 1, column + 1);
                            } catch (IllegalArgumentException exc) {
                                table.addColumnSelectionInterval(0, 0);
                            }
                        }
//                        table.addColumnSelectionInterval(column, column);
                    }
                    fireEditingCanceled(); // jotta palauttaa virheellisest arvosta takaisin
                }
                if (e.getKeyChar() == '\t') {
                    table.clearSelection();
                    try {
                        table.addColumnSelectionInterval(column + 1, column + 1);
                        table.addRowSelectionInterval(row, row);
                    } catch (IllegalArgumentException ex) {
                        table.addColumnSelectionInterval(0, 0);
                        try {
                            table.addRowSelectionInterval(row + 1, row + 1);
                        } catch (IllegalArgumentException exc) {
                            table.addRowSelectionInterval(0, 0);
                        }
//                        table.addColumnSelectionInterval(column, column);
                    }
                    fireEditingCanceled();
                } else if (e.getKeyCode() == KeyEvent.VK_UP ||
                           e.getKeyCode() == KeyEvent.VK_DOWN) {
                    fireEditingCanceled();
                }
                JTextField edit = ((JTextField)e.getComponent()); 
                String s = edit.getText();
                if ( s.equals(currentValue) ) return;
                currentValue = s;
                ok = handleNewValue(table,s);
                if ( ok == null ) {
                    edit.setBackground(Color.white);
                    edit.setToolTipText("");
                    ((ICTable)table).setError(getNr(), getKentta(), null);
                }
                else {
                    edit.setBackground(Color.red);
                    edit.setToolTipText(ok);
                    ((ICTable)table).setError(getNr(), getKentta(), ok);
                }
            }
        });
        cellEditor.addMouseListener(new MouseAdapter() {
            
            @Override
            public void mouseReleased(MouseEvent e) {
                cellEditor.selectAll();
                cellEditor.removeMouseListener(this);
            }
        });
        table.addMouseListener(new MouseAdapter() {
            
            @SuppressWarnings("synthetic-access")
            @Override
            public void mouseClicked(MouseEvent arg0) {
                fireEditingCanceled();
            }
        });
        return cellEditor; 
    }


    /**
     * Ksitelln arvo, joka on muuttunut editoinnin tuloksena
     * @param table taulukko, josta editointi tuli
     * @param s kentn uusi arvo
     * @return null jos kunnossa, muut virheilmoituksia
     */
    protected String handleNewValue(JTable table, String s) {
        String virhe = null;
        ICTable itable = (ICTable)table;
        if ( oliot != null ) virhe = oliot.setAttribute(getNr(), getKentta(), s);
        if ( virhe != null ) return virhe;
        virhe = itable.checkAttribute(getNr(), getKentta(), s);
        if ( virhe == null && itable.isEditorSetValues()) table.setValueAt(s, getIrow(), getIcol());
        return virhe;
    }

    
    /**
     * Asetetaan miss laidassa editoidaan
     * @param cellEditor editoitava kentt
     */
    protected void setAlignment(JTextField cellEditor) {
       cellEditor.setHorizontalAlignment(haling);
    }
    
}
