package fi.jyu.mit.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.RowSorter;
import javax.swing.SwingConstants;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

/**
 * Luokka JTablen "yksinkertaiseksi kyttmiseksi.  Katso mallia: StringTableSuunniteltuKayttotapa
 * @author Vesa Lappalainen
 * @version 24.4.2011
 * TODO: jostakin syyst jumiintuu vlill esim. TABin panamisesta niin, ett
 *       vain sarakkeen lajittelu auttaa ja psee jatkamaan. 
 * TODO: vlill jumiintuu niin ettei editointi toimi jollei paina F2      
 * TODO: vrit eivt kest rivin poistamista
 * TODO: desimaalien mr realiluvuissa
 */
public class StringTable extends JPanel implements IStringTable {

    private final ArrayList<TableEditListener> editListenerList = new ArrayList<TableEditListener>();
    private final ArrayList<TableErrorListener> errorListenerList = new ArrayList<TableErrorListener>();
    private final ArrayList<TableValueListener> valueListenerList = new ArrayList<TableValueListener>();
    private final ArrayList<TableRendererListener> rendererListenerList = new ArrayList<TableRendererListener>();
    private final ArrayList<TableSelectionListener> selectionListenerList = new ArrayList<TableSelectionListener>();
    private boolean enterMoves = true;


    /**
     * @return the enterMoves
     */
    public boolean isEnterMoves() {
        return enterMoves;
    }


    /**
     * @param enterMoves the enterMoves to set
     */
    public void setEnterMoves(boolean enterMoves) {
        this.enterMoves = enterMoves;
    }


    /**
     * @param listener kuuntelija editoinnin muutoksille
     */
    public void addTableEditListener(TableEditListener listener) {
        editListenerList.add(listener);
    }


    /**
     * @param listener kuuntelija editoinnin muutoksille
     */
    public void removeTableEditListener(TableEditListener listener) {
        editListenerList.remove(listener);
    }


    /**
     * @param listener kuuntelija virheille muutoksille
     */
    public void addTableErrorListener(TableErrorListener listener) {
        errorListenerList.add(listener);
    }


    /**
     * @param listener kuuntelija virheille muutoksille
     */
    public void removeTableErrorListener(TableErrorListener listener) {
        errorListenerList.remove(listener);
    }


    /**
     * @param listener kuuntelija virheille muutoksille
     */
    public void addTableValueListener(TableValueListener listener) {
        valueListenerList.add(listener);
    }


    /**
     * @param listener kuuntelija virheille muutoksille
     */
    public void removeTableValueListener(TableValueListener listener) {
        valueListenerList.remove(listener);
    }


    /**
     * @param listener kuuntelija muotoilijalle
     */
    public void addTableRendererListener(TableRendererListener listener) {
        rendererListenerList.add(listener);
    }


    /**
     * @param listener kuuntelija muotoilijalle
     */
    public void removeTableRendererListener(TableRendererListener listener) {
        rendererListenerList.remove(listener);
    }


    /**
     * @param listener kuuntelija editoinnin muutoksille
     */
    public void addTableSelectionListener(TableSelectionListener listener) {
        selectionListenerList.add(listener);
    }


    /**
     * @param listener kuuntelija editoinnin muutoksille
     */
    public void removeTableSelectionListener(TableSelectionListener listener) {
        selectionListenerList.remove(listener);
    }


    /**
     * @see fi.jyu.mit.gui.IStringTable#checkAttribute(int, int, java.lang.Object)
     */
    @Override
    public String checkAttribute(int row, int column, Object s) {
        int r = row; // table.convertRowIndexToModel(row);
        int c = column; // table.convertColumnIndexToModel(column);
        for (TableEditListener listener : editListenerList) {
            String ok = listener.tableEdit(this, r, c, s);
            if ( ok != null ) return ok;
        }
        return null;
    }


    /**
     * Asetetaan virhe olion s takia
     * @param row    milt rivilt virhe tuli
     * @param column mist sarakkeesta virhe tuli
     * @param virhe virheen teksti
     */
    public void setError(int row, int column, String virhe) {
        for (TableErrorListener listener : errorListenerList) {
            listener.tableError(this, row, column, virhe);
        }
    }


    /**
     * Taulukko, jossa omistaja 
     */
    protected static class OTable extends JTable implements ICTable {
        private static final long serialVersionUID = 1L;
        private StringTable owner;
        private boolean editorSetValues = true;


        /**
         * @param owner taulukon omistaja
         */
        public OTable(StringTable owner) {
            super();
            this.owner = owner;
        }


        /**
         * @param owner taulukon omistaja
         * @param model kytettv malli
         */
        public OTable(StringTable owner, DefaultTableModel model) {
            super(model);
            this.owner = owner;
        }


        @Override
        public void setModel(TableModel dataModel) {
            super.setModel(dataModel);
            if ( owner != null ) owner.setCellEditors();
        }


        @Override
        public void setValueAt(Object value, int row, int column) {
            if ( owner != null ) owner.setValueAt(value, row, column);
        }

        @Override
        public Object getValueAt(int row, int column)  {
            if ( owner != null ) return owner.getValueAt(row, column);
            return super.getValueAt(row, column);
        }
        
        @Override
        public String checkAttribute(int row, int column, Object s) {
            if ( owner != null ) return owner.checkAttribute(row, column, s);
            return null;
        }


        @Override
        public void setError(int row, int column, String virhe) {
            if ( owner != null ) owner.setError(row, column, virhe);
        }

        @Override
        public TableCellRenderer getCellRenderer(int row, int column) {
            return owner.getRenderer(row, column, super.getCellRenderer(row, column));
        }


        /**
         * @return muuttaako cell editor suoraan taulukon arvoja vai ei.
         */
        @Override
        public boolean isEditorSetValues() {
            return editorSetValues ;
        }
        
        /**
         * @param value muuttaako cell editor suoraan taulukon arvoja vai ei
         */
        @Override
        public void setEditorSetValues(boolean value) {
            editorSetValues = value;
        }
        
        /**
         * @return kuka omistaa tablen
         */
        public StringTable getOwner() {
            return owner;
        }
        
    }

    private static final long serialVersionUID = 1L;

    private final JScrollPane scrollPane = new JScrollPane();
    /**  Sisinen taulukko joka on periaatteessa JTable */
    protected final OTable table;

    private ArrayList<String> otsikot = new ArrayList<String>();
    private TableObject oliot;
    /** Viitetaulukko kunkin rivin oliolle */
    protected List<Object> riviOliot = new ArrayList<Object>();

    private HashMap<Point, Color> varit = new HashMap<Point, Color>();


    /**
     * Luo uuden ObjectTable:n ja asettaa sisllksi annetut oliot. Sarakkeet nimetn
     * automaattisesti.<br/>
     * @param malli jota kytetn
     */
    public StringTable(final TableObject malli) {
        table = new OTable(this); 
        setKuuntelija();
        table.setRowSelectionAllowed(false);
        table.setAutoCreateRowSorter(true);
        setTableObject(malli);

        // alustetaan paneeli
        setLayout(new BorderLayout(0, 0));
        add(scrollPane);
        scrollPane.setViewportView(table);
        
        setEnterAction();
    }


    /**
     * @see "http://stackoverflow.com/questions/9091208/jtable-enter-key" 
     */
    private final void setEnterAction() {
       @SuppressWarnings("unused")
       EditableCellFocusAction editableCellFocusAction = new EditableCellFocusAction(table, KeyStroke.getKeyStroke("ENTER"));
        
        final String nextCell = "Next cell";
        KeyStroke enter = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(enter, nextCell);
        table.getActionMap().put(nextCell, new EnterAction());
        
        table.getInputMap().put(KeyStroke.getKeyStroke("ENTER"), "handleEnter");
        table.getActionMap().put("handleEnter", new EnterAction());
    }


    private boolean enterGoesNextColumn = true;
    
    
    /**
     * @return if Enter key goes to next columns
     */
    public boolean isEnterGoesNextColumn() {
        return enterGoesNextColumn;
    }


    /**
     * @param goesNextColumn does the enter key go to next column (true) or row
     */
    public void setEnterGoesNextColumn(boolean goesNextColumn) {
        this.enterGoesNextColumn = goesNextColumn;
    }


    /**
     * Mit tapahtuu kun painteaan enteri
     */
    protected void doEnterAction() {
        if ( !enterMoves ) return;
        int c = getTable().getSelectedColumn();
        int r = getTable().getSelectedRow();
        if ( enterGoesNextColumn ) {
          c++;
          if ( c >= getColumnCount() ) { c = 0; r++; }
        } else {
          r++;            
          if ( r >= getRowCount() ) { r = 0; c++; }
        }
        selectCell(r, c);
    }
    
    
    /**
     * Mit tapahtuu kun Enteri painetaan taulukossa
     */
    protected class EnterAction extends AbstractAction {
        private static final long serialVersionUID = 1L;

        @Override
        public void actionPerformed(ActionEvent e) {
            doEnterAction();
        }

    }
    
    
    
    /**
     * Muodostaa uuden tyhjn ObjectTable:n.
     */
    public StringTable() {
        this(new DefaultTableModel());
    }



    /**
     * Muodostaa uuden ObjectTable:n annetulla sisltmodelilla.
     * @param model mit taulukkoon laitetaan sislle
     */
    public StringTable(DefaultTableModel model) {
        table = new OTable(this, model); 
        for (int i=0; i<model.getRowCount(); i++) riviOliot.add(null);
        setKuuntelija(); // TODO: pitisi testata ettei riko toimintaa tll
        table.setAutoCreateRowSorter(true);

        setLayout(new BorderLayout(0, 0));
        add(scrollPane);
        scrollPane.setViewportView(table);
        setEnterAction();
    }


    /**
     * Luokka numeroiden muotoilijoille 
     * @author vesal
     * @version 1.5.2011
     */
    protected static class NumberRenderer extends DefaultTableCellRenderer.UIResource {
        private static final long serialVersionUID = 1L;


        /**
         * Alustetaan oikeaan reunaan
         */
        public NumberRenderer() {
            super();
            setHorizontalAlignment(SwingConstants.RIGHT);
        }
    }


    /**
     * Luokka merkkijonojen muotoilijoille 
     * @author vesal
     * @version 1.5.2011
     */
    protected static class StringRenderer extends DefaultTableCellRenderer.UIResource {
        private static final long serialVersionUID = 1L;


        /**
         * Alustetaan oikeaan reunaan
         */
        public StringRenderer() {
            super();
            setHorizontalAlignment(SwingConstants.LEFT);
        }
    }


    /**
     * Luokka reaaliluvun muotoilemiseksi 2:lla desimaalilla
     * @author vesal
     * @version 1.5.2011
     */
    protected static class DoubleRenderer extends NumberRenderer {
        private static final long serialVersionUID = 1L;
        NumberFormat formatter;


        /** Alustetaan oikeaan reunaan */
        public DoubleRenderer() {
            super();
        }


        @Override
        public void setValue(Object value) {
            if ( formatter == null ) {
                formatter = NumberFormat.getInstance();
            }
            //setText((value == null) ? "" : formatter.format(value));
            //            setText((value == null) ? "" : value.toString());
            if ( value instanceof Double ) {
                setText(String.format("%3.2f", value).replace(',', '.'));
                return;
            }
            setText((value == null) ? "" : value.toString());
        }
    }

    private DoubleRenderer doubleRender = new DoubleRenderer();
    private StringRenderer stringRender = new StringRenderer();
    private NumberRenderer numberRender = new NumberRenderer();


    /**
     * Palautetaan solun muotoilija
     * @param row  mille riville
     * @param column mille sarakkeelle
     * @param def oletusmuotoilija jota kytetn jos omaa ei ole
     * @return solun muotoilija
     */
    protected TableCellRenderer getRenderer(int row, int column, TableCellRenderer def) {
        DefaultTableCellRenderer renderer = (DefaultTableCellRenderer)def;
        renderer = stringRender;
        if ( table.getColumnClass(column) == Double.class ) renderer = doubleRender;
        if ( table.getColumnClass(column) == Integer.class ) renderer = numberRender;
        int nr = table.convertRowIndexToModel(row);
        int kentta = table.convertColumnIndexToModel(column);
        Point sijainti = new Point(nr, kentta);
        Color col = varit.get(sijainti);
        if ( col != null ) {
            renderer.setBackground(col);
            return renderer;
        }
        //return super.getCellRenderer(row, column);
        renderer.setBackground(table.getBackground());
        for (TableRendererListener listener : rendererListenerList)
            renderer = listener.tableRenderer(this, renderer, nr, kentta);
        return renderer;
    }


    /**
     * Muodostaa uuden ObjectTable:n ja alustaa sen annetuilla mrill tyhji rivej
     * ja sarakkeita.
     * @param riveja montako rivi halutaan
     * @param sarakkeita montako saraketta halutaan
     */
    public StringTable(int riveja, int sarakkeita) {
        this(new DefaultTableModel(riveja, sarakkeita));
    }


    /**
     * Muodostaa uuden ObjectTable:n ja alustaa sen annetuilla sisllll ja
     * sarakkeiden nimill.
     * @param sisalto mit taulukkoon aluksi laitetaan
     * @param nimet sarakkeiden nimet
     */
    public StringTable(Object[][] sisalto, Object[] nimet) {
        this(new DefaultTableModel(sisalto, nimet));
    }


    /**
     * Muodostaa uuden StringTable:n ja alustaa sen annetuilla sarakkeiden nimill ja
     * halutulla mrll tyhji rivej.
     * @param nimet sarakkeiden nimet
     * @param riveja montako rivi halutaan
     */
    public StringTable(Object[] nimet, int riveja) {
        this(new DefaultTableModel(nimet, riveja));
    }


    // sarakkeiden ksittely

    /**
     * Lis taulukkoon annetun sarakkeen, kyt jos haluat luoda oman saraketyypin.
     * @param column millainen sarake listn
     */
    public void addColumn(TableColumn column) {
        addColumn(column, "new column");
    }


    /**
     * Lis taulukkoon annetun sarakkeen, kyt jos haluat luoda oman saraketyypin.
     * @param column millainen sarake listn
     * @param header sarakkeen otsikko
     */
    public void addColumn(TableColumn column, String header) {
        int i = table.getModel().getColumnCount();
        otsikot.add(header);
        ((DefaultTableModel)table.getModel()).addColumn(otsikot.get(i));
        table.addColumn(column);
        setCellEditors();
    }


    // rivien ksittely

    /**
     * Lis uuden rivin keskelle
     */
    private void addRow(int rivi) {
        ((DefaultTableModel)table.getModel()).insertRow(rivi, new Object[getColumnCount()]);
        riviOliot.add(rivi, null);
        if ( oliot != null && rivi < oliot.getObjectCount() )
            for (int sarake = 0; sarake < oliot.getAttributeCount(); sarake++) {
                setValueAt(oliot.getAttribute(rivi, sarake), rivi, sarake);
            }
        //setCellEditors();
    }


    /**
     * Yritt list taulukkoon rivin.
     * @return listyn rivin indeksi
     */
    public int addRow() {
        ((DefaultTableModel)table.getModel()).addRow(new Object[getColumnCount()]);
        if ( oliot != null && oliot.addObject() ) {
            int rivi = table.getRowCount() - 1;
            for (int sarake = 0; sarake < oliot.getAttributeCount(); sarake++) {
                setValueAt(oliot.getAttribute(rivi, sarake), rivi, sarake);
            }
        }
        riviOliot.add(null);
        // setCellEditors();
        return getRowCount() - 1;
    }


    /**
     * Poistaa pyydetyn rivin.
     * @param rivi poistettavan rivin indeksi
     * @return onnistuiko rivin poisto
     * @throws IndexOutOfBoundsException jos annetaan huono rivi
     */
    public boolean removeRow(int rivi) {
        if ( oliot == null ) {
            ((DefaultTableModel)table.getModel()).removeRow(rivi);
            if ( rivi < riviOliot.size() ) riviOliot.remove(rivi);
            /*
            for (int j=0; j < koordit.size(); j++) {
                if (koordit.get(j).x == rivi) {
                    koordit.remove(j);
                    varit.remove(j);
                } else if (koordit.get(j).x > rivi) {
                    koordit.get(j).x--;
                }
            }
            */
            return true;
        }
        if ( oliot.removeObject(rivi) ) {
            ((DefaultTableModel)table.getModel()).removeRow(rivi);
            if ( rivi < riviOliot.size() ) riviOliot.remove(rivi);
            /*
            for (int j=0; j < koordit.size(); j++) {
                if (koordit.get(j).x == rivi) {
                    koordit.remove(j);
                    varit.remove(j);
                } else if (koordit.get(j).x > rivi) {
                    koordit.get(j).x--;
                }
            }
            */
            return true;
        }
        return false;
    }


    /**
     * Poistaa kaikki rivit taulukosta
     */
    public void clear() {
        //for (int i = getRowCount() - 1; i >= 0; i--)
        //    removeRow(i);
        ((DefaultTableModel)(table.getModel())).setRowCount(0);
        
        // Seuraava hsskk jotta saadaan lajittelu pois tyhjentmisen jlkeen 
        RowSorter<? extends TableModel> rs = table.getRowSorter();
        rs.toggleSortOrder(0);
        if ( table.getAutoCreateRowSorter() ) table.setAutoCreateRowSorter(true);
        
        // Seuraava poistaa mm. lajittelun mutta valitettavasti mys sarakkeiden leveydet
        //((DefaultTableModel)(table.getModel())).fireTableStructureChanged();
        riviOliot.clear();
        varit.clear();
    }


    /**
     * Poistaa valitun rivin.
     * @return onnistuiko rivin poisto
     */
    public boolean removeSelectedRow() {
        try {
            return removeRow(table.convertRowIndexToModel(table.getSelectedRow()));
        } catch (IndexOutOfBoundsException e) {
            return false;
        }
    }

    
    /**
     * @return valitun rivin numero
     */
    @Override
    public int getSelectedRow() {
        return table.getSelectedRow();
    }
    

    /**
     * @return valitun sarakkeen numero
     */
    @Override
    public int getSelectedColumn() {
        return table.getSelectedColumn();
    }
    

    /**
     * @return valitun rivin numero
     */
    @Override
    public int getSelectedRowAtModel() {
        return convertRowIndexToModel(table.getSelectedRow());
    }
    

    /**
     * @return valitun sarakkeen numero
     */
    @Override
    public int getSelectedColumnAtModel() {
        return convertColumnIndexToModel(table.getSelectedColumn());
    }
    

    // muuta

    /**
     * Lis muottiin kuuntelijan kuuntelemaan sen muutoksia, ett ne voidaan
     * pivitt omiinkin tietoihin sit mukaa kuin tulevat. Vaaditaan sen verran
     * perehtymist, ett kyttj osaa itse muotoilla mit on kuuntelemassa.
     * @param l listtv kuuntelija
     */
    public void addTableModelListener(TableModelListener l) {
        table.getModel().addTableModelListener(l);
    }


    /**
     * Poistaa muotista kuuntelijan, jota ei en tarvita.
     * @param l poistettava kuuntelija
     */
    public void removeTableModelListener(TableModelListener l) {
        table.getModel().removeTableModelListener(l);
    }


    @Override
    public synchronized void addFocusListener(FocusListener l) {
        table.addFocusListener(l);
    }


    @Override
    public synchronized void removeFocusListener(FocusListener l) {
        table.addFocusListener(l);
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#setCellColor(int, int, java.awt.Color)
     */
    @Override
    public void setCellColor(int row, int column, final Color color) {
        //int nr = table.convertRowIndexToModel(row);
        //int kentta = table.convertColumnIndexToModel(column);
        //Point sijainti = new Point(nr, kentta);
        Point sijainti = new Point(row, column);
        varit.put(sijainti, color);
        ((DefaultTableModel)table.getModel()).fireTableCellUpdated(row, column);
    }


    /**
     * Palauttaa sarakkeiden nimet taulukkona.
     * @return sarakkeiden nimet
     */
    public String[] getColumnNames() {
        return otsikot.toArray(new String[0]);
    }


    /**
     * Etsii kyseisen otsikon omaavan sarakkeen numeron.
     * @param columnName sarakkeen nimi
     * @return sarakkeen numero
     */
    public int findColumn(String columnName) {
        return otsikot.indexOf(columnName);
    }


    /**
     * Vaihtaa pyydetyn sarakkeen otsikon halutuksi.
     * @param column sarakkeen indeksi
     * @param newHeader sarakkeen uusi otsikko
     * @throws IndexOutOfBoundsException jos indeksi on huono
     */
    public void setHeader(int column, String newHeader) {
        otsikot.set(column, newHeader);
    }


    /**
     * @return taulukon sarakkeiden mr
     */
    public int getColumnCount() {
        return table.getModel().getColumnCount();
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getRowCount()
     */
    @Override
    public int getRowCount() {
        return table.getModel().getRowCount();
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getValueAtModel(int, int)
     */
    @Override
    public Object getValueAtModel(int row, int column) {
        int r = row; // table.convertRowIndexToModel(row);
        int c = column; // table.convertColumnIndexToModel(column);
        if ( valueListenerList.size() != 0 ) {
            return valueListenerList.get(0).tableValue(this, r, c);
        }
        Object value = table.getModel().getValueAt(r, c);
        if ( value == null ) return null;
        return value;
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getValueAt(int, int)
     */
    @Override
    public Object getValueAt(int row, int column) {
        return getValueAtModel(table.convertRowIndexToModel(row), table.convertColumnIndexToModel(column));
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getSelectedValue()
     */
    @Override
    public Object getSelectedValue() {
        return getValueAt(table.getSelectedRow(), table.getSelectedColumn());
    }


    /**
     * Asettaa annetun arvon haluttuun alkioon taulukossa.
     * Koordinaatit mallin koordinaatteja.
     * @param row mille riville asetetaan
     * @param column mihin sarakkeeseen asetetaan
     * @param value mik arvo alkioon asetetaan
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    public void setValueAtModel(Object value, int row, int column) {
        table.getModel().setValueAt(value, row, column);
        return;
    }


    /**
     * Asettaa annetun arvon haluttuun alkioon taulukossa.
     * Koordinaatit taulukon koordinaatteja.
     * @param row mille riville asetetaan
     * @param column mihin sarakkeeseen asetetaan
     * @param value mik arvo alkioon asetetaan
     * @throws ArrayIndexOutOfBoundsException jos annetaan huono rivi tai sarake
     */
    public void setValueAt(Object value, int row, int column) {
        setValueAtModel(value, table.convertRowIndexToModel(row), table.convertColumnIndexToModel(column));
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getObjectAt(int)
     */
    @Override
    public Object getObjectAt(int row) {
       if ( row >= riviOliot.size() ) return null; 
       return riviOliot.get(row);
    }


    /**
     * Kiinnitt riviin olion jota kytetn, jos soluun ei ole erikseen liimattu
     * oliota.
     * Koordinaatit mallin koordinaatteja
     * @param obj liimattava olio
     * @param row mille riville
     * @throws IndexOutOfBoundsException huono sijainti riville
     */
    public void setObjectAt(Object obj, int row) {
        for (int i=riviOliot.size(); i<=row; i++) riviOliot.add(null);
        riviOliot.set(row, obj);
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getModel()
    @Override
    public DefaultTableModel getModel() {
        return (DefaultTableModel)table.getModel();
    }
     */


    /*
     * Asettaa taulukkoon muotin
     * @param model taulukkoon laitettava muotti
    public void setModel(TableModel model) {
        table.setModel(model);
    }
     */


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getTable()
     */
    @Override
    public JTable getTable() {
        // todettu ett kyttj rikkoo toiminnan heti kyttessn tt
        return table;
    }


    
    
    /**
     * Palauttaa tableModelin joka ksittelee tll taulukolle sopivalla tavalla.
     * Tm on tarkoitettu perittvksi jos taulukosta peritn muunlaisia taulukoita.
     * @param sarakeTyypit lista sarkkaeiden tyypeist
     * @return talukolle tableModel
     */
    protected DefaultTableModel getNewTableModel(final ArrayList<Class<?>> sarakeTyypit) {
        return new DefaultTableModel(
                new Object[0][sarakeTyypit.size()],
                getColumnNames()) {
            private static final long serialVersionUID = 1L;
            Class<?>[] columnTypes = sarakeTyypit.toArray(new Class<?>[0]);


            @Override
            public Class<?> getColumnClass(int columnIndex) {
                return columnTypes[columnIndex];
            }


            @Override
            public Object getValueAt(int row, int column) {
                Object value = super.getValueAt(row, column);
                if ( value == null ) return null;
                return value;
            }
        };
    }


    /**
     * @param sisalto asetettava sislt
     */
    public void setTableObject(final TableObject sisalto) {
        oliot = sisalto;
        // alustetaan sarakkeet
        final ArrayList<Class<?>> sarakeTyypit = new ArrayList<Class<?>>();
        for (int i = 0; i < oliot.getAttributeCount(); i++) {
            sarakeTyypit.add(oliot.getClass(i));
            otsikot.add(oliot.getHeader(i));
        }
        table.setModel(getNewTableModel(sarakeTyypit));
        /*
        new DefaultTableModel(
        new Object[0][sarakeTyypit.size()],
        getColumnNames()
        ) {
            private static final long serialVersionUID = 1L;
            Class<?>[] columnTypes = sarakeTyypit.toArray(new Class<?>[0]);


            @Override
            public Class<?> getColumnClass(int columnIndex) {
                return columnTypes[columnIndex];
            }


            @Override
            public Object getValueAt(int row, int column) {
                Object value = super.getValueAt(row, column);
                if ( value == null ) return null;
                return value;
            }
        }
        );
        */
        setCellEditors();
        // alustetaan rivit
        for (int j = 0; j < oliot.getObjectCount(); j++) {
            addRow(j);
        }
    }


    /**
     * Asettaa taulukkoon sopivat solunmuokkaajaoliot.
     */
    protected void setCellEditors() {
        for (int i = 0; i < table.getModel().getColumnCount(); i++) {
            Class<?> luokka = table.getColumnClass(i);
            StringCellEditor editor;
            if ( luokka == Integer.class )     editor = new IntegerCellEditor(oliot);
            else if ( luokka == Double.class ) editor = new DoubleCellEditor(oliot);
            else                               editor = new StringCellEditor(oliot);
            table.getColumnModel().getColumn(i).setCellEditor(editor);
            editor.setFont(table.getFont());
            DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
            for (TableRendererListener listener : rendererListenerList)
                renderer = listener.tableRenderer(this, renderer, 0, i);
            editor.setHaling(renderer.getHorizontalAlignment());
            editor.addCellEditorListener(new CellEditorListener() {
                @Override
                public void editingStopped(ChangeEvent arg0) {
                    selectionChanged();
                }
                
                @Override
                public void editingCanceled(ChangeEvent arg0) {
                    selectionChanged();
                }
            });
        }
        
        table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                selectionChanged();
            }
        });
        
        table.getColumnModel().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                selectionChanged();
            }
        });
        

    }


    /**
     * Tt kutsutaan kun valinta vaihtuu 
     */
    protected void selectionChanged() {
        for (TableSelectionListener listener : selectionListenerList )
            listener.selectionChanged(this, getSelectedRow(), getSelectedColumn());
    }


    /**
     * kuuntelijat nppimille
     */
    protected void setKuuntelija() {
        table.addKeyListener(new KeyAdapter() {

            @Override
            public void keyTyped(KeyEvent e) {
                if ( e.getKeyCode() != '\t' &&  
                        e.getKeyCode() != '\n' &&
                        e.getKeyCode() != KeyEvent.VK_LEFT &&
                        e.getKeyCode() != KeyEvent.VK_UP &&
                        e.getKeyCode() != KeyEvent.VK_RIGHT &&
                        e.getKeyCode() != KeyEvent.VK_DOWN ) {
                    int row = table.getSelectedRow();
                    int column = table.getSelectedColumn();
                    if ( row < 0 || column < 0 ) return;
                    table.setCellEditor(table.getCellEditor(row, column));
                    Component editor = table.getCellEditor().
                            getTableCellEditorComponent(table, table.getModel().
                                    getValueAt(table.convertRowIndexToModel(row),
                                            table.convertColumnIndexToModel(column)), true,
                                    row, column);
                    editor.setBounds(table.getCellRect(row, column, false));
                    editor.requestFocusInWindow();
                    editor.setVisible(true);
                }
            }
        });
    }


    /**
     * Muuttaa taulukon rivin indeksinmallin riviksi
     * @param row taulukon rivi
     * @return vastaava rivinumero mallin rivin
     */
    @Override
    public int convertRowIndexToModel(int row) {
        if ( row < 0 ) return -1;
        return table.convertRowIndexToModel(row);
    }

    /**
     * Muuttaa taulukon sarakkeen indeksi mallin sarakkeeksi
     * @param col taulukon sarake
     * @return vastaava rivinumero mallin sarakkeena
     */
    @Override
    public int convertColumnIndexToModel(int col) {
        if ( col < 0 ) return -1; 
        return table.convertColumnIndexToModel(col);
    }


    /**
     * @see fi.jyu.mit.gui.IStringTable#getSelectedObject()
     */
    @Override
    public Object getSelectedObject() {
        int row = getSelectedRow();
        if ( row < 0 ) return null;
        return getObjectAt(convertRowIndexToModel(row));
    }


    @Override
    public void selectCell(int row, int col) {
        /*
        int r = row;
        int c = col;
        int mrow = getRowCount()-1;
        int mcol = getColumnCount()-1;
        if ( mrow < r) r = mrow;
        if ( mcol < c) c = mcol;
        // table.requestFocus();
        if ( c < 0 || r < 0 ) { table.clearSelection(); return; } 
        table.setRowSelectionInterval(r, r);
        table.setColumnSelectionInterval(c, c);
        */
        selectCell(row, col,false);
    }


    /**
     * @param row mik rivi aktivoidaan
     * @param col mik sarake aktivoidaan
     * @param getFocus vaaditaanko focus
     */
    public void selectCell(int row, int col, boolean getFocus) {
        int r = row;
        int c = col;
        int mrow = getRowCount()-1;
        int mcol = getColumnCount()-1;
        if ( mrow < r) r = mrow;
        if ( mcol < c) c = mcol;
        if (getFocus) table.requestFocus();
        if ( c < 0 || r < 0 ) { table.clearSelection(); return; } 
        table.setRowSelectionInterval(r, r);
        table.setColumnSelectionInterval(c, c);
    }


    /* (non-Javadoc)
     * @see fi.jyu.mit.gui.IStringTable#getScrollPane()
     */
    @Override
    public JScrollPane getScrollPane() {
        return scrollPane;
    }

    
    /**
     * @return muuttaako cell editor suoraan taulukon arvoja vai ei.
     */
    public boolean isEditorSetValues() {
        return table.isEditorSetValues();
    }
    
    /**
     * @param value muuttaako cell editor suoraan taulukon arvoja vai ei
     */
    public void setEditorSetValues(boolean value) {
        table.setEditorSetValues(value);
    }
    
}
