package fi.jyu.mit.gui;

import java.security.InvalidParameterException;
import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

/**
 * Luotu 23.3.2011.
 * Taulukkopohja merkkijonoille.
 * @author Samu Tuominen
 * @version 26.3.2011
 */
public class TableModelString extends AbstractTableModel {
    
    private static final long serialVersionUID = 1L;
    
    private ArrayList<String> otsikot = new ArrayList<String>();
    private Matriisi<String> taulukko;
    
    
    /**
     * Luo uuden tyhjn taulukkopojan.
     */
    public TableModelString() {
        taulukko = new Matriisi<String>();
    }
    
    
    /**
     * Luo uuden tyhjn taulukkopojan halutulla mrll rivej ja sarakkeita.
     * @param riveja rivien mr
     * @param sarakkeita sarakkeiden mr
     */
    public TableModelString(int riveja, int sarakkeita) {
        this();
        taulukko.lisaaRiveja(riveja);
        taulukko.lisaaSarakkeita(sarakkeita);
        for (int i=0; i < sarakkeita; i++)
            otsikot.add("uusi sarake");
    }
    
    
    /**
     * Luo uuden taulukon annetulla sisllll ja nime sarakkeiden otsikot
     * automaattisesti (huonot nimet).
     * @param sisalto taulukkoon asetettava sislt.
     */
    public TableModelString(Matriisi<String> sisalto) {
        for (int i=0; i < sisalto.getSarakkeita(); i++) otsikot.add("uusi sarake");
        taulukko = sisalto.clone();
    }
    
    
    /**
     * Luo uuden taulukkomuotin annetuilla sisllll ja sarakkeiden otsikoilla.
     * @param sisalto taulukkoon asetettava sislt.
     * @param nimet sarakkeiden otsikot
     */
    public TableModelString(Matriisi<String> sisalto, String[] nimet) {
        if (nimet.length != sisalto.getSarakkeita())
            throw new InvalidParameterException("Taulukossa tytyy olla yht monta " +
                    "otsikkoa ja saraketta!");
        for (int i=0; i < nimet.length; i++) otsikot.add(nimet[i]);
        taulukko = sisalto.clone();
    }
    
    
    /**
     * Luo uuden taulukon annetulla sisllll ja nime sarakkeiden otsikot
     * automaattisesti (huonot nimet).
     * @param sisalto taulukkoon asetettava sislt.
     */
    public TableModelString(String[][] sisalto) {
        for (int i=0; i < sisalto[0].length; i++) otsikot.add("uusi sarake");
        taulukko = new Matriisi<String>(sisalto);
    }
    
    
    /**
     * Luo uuden taulukkomuotin annetuilla sisllll ja sarakkeiden otsikoilla.
     * @param sisalto taulukkoon asetettava sislt.
     * @param nimet sarakkeiden otsikot
     */
    public TableModelString(String[][] sisalto, String[] nimet) {
        if (nimet.length != sisalto[0].length)
            throw new InvalidParameterException("Taulukossa tytyy olla yht monta " +
            		"otsikkoa ja saraketta!");
        for (int i=0; i < nimet.length; i++) otsikot.add(nimet[i]);
        taulukko = new Matriisi<String>(sisalto);
    }
    
    
    /**
     * Luo uuden taulukon annetuilla sarakkeiden otsikoilla ja rivimrll.
     * @param nimet sarakkeiden otsikot
     * @param riveja rivien mr
     */
    public TableModelString(String[] nimet, int riveja) {
        for (int i=0; i < nimet.length; i++) otsikot.add(nimet[i]);
        taulukko = new Matriisi<String>();
        taulukko.lisaaRiveja(riveja);
        taulukko.lisaaSarakkeita(nimet.length);
    }


    @Override
    public int getColumnCount() {
        return taulukko.getSarakkeita();
    }


    @Override
    public int getRowCount() {
        return taulukko.getRiveja();
    }
    
    
    /**
     * @throws IndexOutOfBoundsException huonot indeksit
     */
    @Override
    public String getValueAt(int rivi, int sarake) throws IndexOutOfBoundsException {
        return taulukko.haeArvo(rivi, sarake);
    }


    /**
     * Asettaa annetun arvon haluttuun alkioon.
     * @param arvo mik arvo alkioon asetetaan
     * @param rivi mille riville asetetaan
     * @param sarake mihin sarakkeeseen asetetaan
     * @return onnistuiko arvon asettaminen
     */
    public boolean asetaArvo(String arvo, int rivi, int sarake) {
        if (taulukko.asetaArvo(arvo, rivi, sarake)) {
            fireTableCellUpdated(rivi, sarake);
            return true;
        }
        return false;
    }
    
    
    /**
     * Suositellaan kyttmn mieluummin metodia asetaArvo, koska siin arvoa ei
     * tarvitse ensin muuttaa merkkijonoksi.
     * @throws IndexOutOfBoundsException huono sijainti
     */
    @Override
    public void setValueAt(Object arvo, int rivi, int sarake)
            throws IndexOutOfBoundsException {
        String sisalto = arvo.toString();
        if (asetaArvo(sisalto, rivi, sarake))
            return;
        throw new IndexOutOfBoundsException("Huonot indeksit!");
    }
    
    
    /**
     * Vaihtaa pyydetyn sarakkeen otsikon halutuksi.
     * @param monesko sarakkeen indeksi
     * @param uusiOtsikko sarakkeen uusi otsikko
     * @return onnistuiko otsikon vaihto
     */
    public boolean vaihdaOtsikko(int monesko, String uusiOtsikko) {
        try {
            otsikot.set(monesko, uusiOtsikko);
            fireTableStructureChanged();
            return true;
        } catch (IndexOutOfBoundsException e) {
            return false;
        }
    }
    

    /**
     * @throws IndexOutOfBoundsException huonot indeksit
     */
    @Override
    public String getColumnName(int sarake) throws IndexOutOfBoundsException {
        return otsikot.get(sarake);
    }
    
    
    /**
     * @return sarakkeiden nimet
     */
    public String[] getColumnNames() {
        return otsikot.toArray(new String[0]);
    }
    
    @Override
    public int findColumn(String columnName) {
        return otsikot.indexOf(columnName);
    }
    
    
    /**
     * Palauttaa true, jos alkio on kytettviss.
     */
    @Override
    public boolean isCellEditable(int rivi, int sarake) {
        if (rivi < 0 || rivi >= taulukko.getRiveja()) return false;
        if (sarake < 0 || sarake >= taulukko.getSarakkeita()) return false;
        return true;
    }
    
// sarakkeiden ksittely

    /**
     * Lis annetun mrn tyhji sarakkeita matriisiin.
     * @param maara montako tyhj saraketta halutaan
     * @param nimet sarakkeiden otsikot
     * @return onnistuiko sarakkeiden lisminen
     */
    public boolean lisaaSarakkeita(int maara, String[] nimet) {
        return lisaaSarakkeita(maara, taulukko.getSarakkeita(), nimet);
    }
    
    
    /**
     * Lis annetun mrn tyhji sarakkeita matriisiin.
     * @param maara montako tyhj saraketta halutaan
     * @param minne monennenko sarakkeen paikalle laitetaan
     * @param nimet sarakkeiden otsikot
     * @return onnistuiko sarakkeiden lisminen
     */
    public boolean lisaaSarakkeita(int maara, int minne, String[] nimet) {
        if (nimet.length != maara) return false;
        if (taulukko.lisaaSarakkeita(maara, minne)) {
            for (int i=0; i < maara; i++) otsikot.add(minne + i, nimet[i]);
            fireTableStructureChanged();
            return true;
        }
        return false;
    }
    
    
    /**
     * Lis annetun sarakkeen matriisin rivien phn.
     * @param sarake listtv sarake
     * @param nimi sarakkeen otsikko
     * @return onnistuiko sarakkeen lisminen
     */
    public boolean lisaaSarake(String[] sarake, String nimi) {
        return lisaaSarake(sarake, taulukko.getSarakkeita(), nimi);
    }
    

    /**
     * Lis annetun sarakkeen matriisiin halutulle sarakkeelle ja siirt loppuja
     * sarakkeita eteenpin.
     * @param sarake listtv sarake
     * @param minne monennenko sarakkeen paikalle laitetaan
     * @param nimi sarakkeen otsikko
     * @return onnistuiko sarakkeen lisminen
     */
    public boolean lisaaSarake(String[] sarake, int minne, String nimi) {
        if (taulukko.lisaaSarake(sarake, minne)) {
            otsikot.add(minne, nimi);
            fireTableStructureChanged();
            return true;
        }
        return false;
    }
    
    
    /**
     * Poistaa pyydetyn sarakkeen.
     * @param i poistettavan sarakkeen indeksi
     * @return onnistuiko sarakkeen poistaminen
     */
    public boolean poistaSarake(int i) {
        if (taulukko.poistaSarake(i)) {
            otsikot.remove(i);
            fireTableStructureChanged();
            return true;
        }
        return false;
    }
    
    
    /**
     * Poistaa pyydetyt sarakkeet.
     * @param alku monennestako sarakkeesta poistaminen aloitetaan
     * @param loppu monesko sarake on ensimminen, joka silytetn
     * @return onnistuiko sarakkeiden poistaminen
     */
    public boolean poistaSarakkeita(int alku, int loppu) {
        if (taulukko.poistaSarakkeita(alku, loppu)) {
            for (int i=(loppu - 1); i >= alku; i--) otsikot.remove(i);
            fireTableStructureChanged();
            return true;
        }
        return false;
    }
    
// rivien ksittely
    
    /**
     * Lis annetun mrn tyhji rivej matriisiin.
     * @param maara montako tyhj rivi halutaan
     * @return onnistuiko rivien lisminen
     */
    public boolean lisaaRiveja(int maara) {
        if (taulukko.lisaaRiveja(maara)) {
            fireTableRowsInserted(getRowCount() - maara, getRowCount()-1);
            return true;
        }
        return false;
    }
    
    
    /**
     * Lis annetun mrn tyhji rivej matriisiin.
     * @param maara montako tyhj rivi halutaan
     * @param minne monennenko rivin paikalle laitetaan
     * @return onnistuiko rivien lisminen
     */
    public boolean lisaaRiveja(int maara, int minne) {
        if (taulukko.lisaaRiveja(maara, minne)) {
            fireTableRowsInserted(minne, minne + maara);
            return true;
        }
        return false;
    }
    
    
    /**
     * Lis annetun rivin matriisin sarakkeiden phn.
     * @param rivi listtv rivi
     * @return onnistuiko rivin lisminen
     */
    public boolean lisaaRivi(String[] rivi) {
        if (taulukko.lisaaRivi(rivi)) {
            fireTableRowsInserted(getRowCount() - 1, getRowCount());
            return true;
        }
        return false;
    }
    
    
    /**
     * Lis annetun rivin matriisiin halutulle riville ja siirt loppuja rivej
     * eteenpin.
     * @param rivi listtv rivi
     * @param minne monennenko rivin paikalle laitetaan
     * @return onnistuiko rivin lisminen
     */
    public boolean lisaaRivi(String[] rivi, int minne) {
        if (taulukko.lisaaRivi(rivi, minne)) {
            fireTableRowsInserted(minne, minne + 1);
            return true;
        }
        return false;
    }
    
    
    /**
     * Poistaa pyydetyn rivin.
     * @param i poistettavan rivin indeksi
     * @return onnistuiko rivin poisto
     */
    public boolean poistaRivi(int i) {
        if (taulukko.poistaRivi(i)) {
            fireTableRowsDeleted(i, i + 1);
            return true;
        }
        return false;
    }
    
    
    /**
     * Poistaa pyydetyt rivit.
     * @param alku monennestako rivist poistaminen aloitetaan
     * @param loppu monesko rivi on ensimminen, joka silytetn
     * @return onnistuiko rivien poisto
     */
    public boolean poistaRiveja(int alku, int loppu) {
        if (taulukko.poistaRiveja(alku, loppu)) {
            fireTableRowsDeleted(alku, loppu-1);
            return true;
        }
        return false;
    }
    
}
