package fi.jyu.mit.gui;

import java.io.*;

import javax.swing.JTextArea;

/**
 * Simple way to "print" to a JTextArea; just say
 * PrintWriter out = new PrintWriter(new TextAreaWriter(myTextArea));
 * Then out.println() et all will all appear in the TextArea.
 * 
 * Source: http://javacook.darwinsys.com/new_recipes/14.9betterTextToTextArea.jsp
 */
public final class TextAreaWriter extends Writer {

    private final JTextArea textArea;

    /**
     * @param textArea area where to write
     * @example
     * <pre name="test">
     * #import java.io.*;
     * #import javax.swing.*;
     *   JTextArea text = new JTextArea();
     *   PrintWriter tw = new PrintWriter(new TextAreaWriter(text));
     *   tw.print("Hello");
     *   tw.print(" ");
     *   tw.print("world!");
     *   text.getText() === "Hello world!";
     *   text.setText("");
     *   tw.println("Hello");
     *   tw.println("world!");
     *   text.getText() =R= "Hello\\r?\\nworld!\\r?\\n";
     * </pre>
     */
    public TextAreaWriter(final JTextArea textArea) {
        this.textArea = textArea;
    }

    @Override
    public void flush(){ /* */ }
    
    @Override
    public void close(){ /* */ }

    @Override
    public void write(char[] cbuf, int off, int len) throws IOException {
        textArea.append(new String(cbuf, off, len));
    }
    

    /**
     * Factory method for creating a PrintWriter to print to selected TextArea
     * @param textArea area where to print
     * @return created PrintWriter ready to print to TextArea
     * @example
     * <pre name="test">
     * #import java.io.*;
     * #import javax.swing.*;
     *   JTextArea text = new JTextArea();
     *   PrintWriter tw = TextAreaWriter.getTextPrintWriter(text);
     *   tw.print("Hyv");
     *   tw.print(" ");
     *   tw.print("piv!");
     *   text.getText() === "Hyv piv!";
     * </pre>
     */
    public static PrintWriter getTextPrintWriter(JTextArea textArea) {
        return new PrintWriter(new TextAreaWriter(textArea)); 
    }

}

