package fi.jyu.mit.gui.example;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import fi.jyu.mit.gui.IStringTable;
import fi.jyu.mit.gui.StringTable;

import javax.swing.table.DefaultTableModel;
import fi.jyu.mit.gui.TableEditListener;
import fi.jyu.mit.gui.TableErrorListener;

import javax.swing.JLabel;
import javax.swing.UIManager;

import fi.jyu.mit.gui.TableSelectionListener;

/**
 * Esimerkki StringTablen kytst. 
 * <pre>
 * 1) Kyttj laittaa lomakkeele StringTablen
 * 2) Sopiva mr sarakkeita taulukkoon ja sinne oikeat tyypit 
 *    (taulukon alta lytyy getTable josta voi st Modelia)
 * 3) Koodissa sopivassa kohti listn data taulukkoon, ks: alkiotTaulukkoon()
 * 4) Kun taulukkoa editoidaan, kuunnellaan tapahtumaa tableEdit, 
 *       ks: addTableEditListener , checkNewValue
 *    Tss voidaan kysy omalta data-olioilta kelpaako arvo ja 
 *    jos ei kelpaa, palautetaan virheilmoitus.
 *    Kokeile esim. laittaa yli 400 op.
 * 5) Jos virheen haluaa nytt muuallakin kuin taulukossa punaisella, 
 *    voi tehd vaikka virheLabelin, jonka arvoa muuttaa tapahtumasta 
 *    tableError, ks:  addTableErrorListener,  setError
 * 6) Uuden data-alkion lisyksess helpointa on list se oikeaan 
 *    data-taulukkoon (ks: lisaaOppilas) ja sitten rakentaa taulukko 
 *    uudelleen.  Onnistuu mys jos lis alkion omaan dataan ja samalla
 *    taulukkoon uuden rivin.
 * 7) Poistaminen on vastaavasti helpointa tehd niin, ett poisto 
 *    omasta datasta ja sitten taulukon uudelleen rakentaminen.   
 * 8) Kullekin solulle voi halutessaan laittaa eri vrin kutsulla tyyliin:
 *       table.setCellColor(2, 1, Color.YELLOW);      
 * </pre>
 * @author Vesa Lappalainen
 * @version 24.4.2011
 */
public class StringTableSuunniteltuKayttotapa extends JFrame {

    private static final long serialVersionUID = 1L;
    private JPanel contentPane;
    private final StringTable table;
    private final JPanel panelValikot = new JPanel();
    private JButton btnPoistaValittuRivi;
    private JButton btnHaeArvo;
    private JButton btnTulostaKaikki;
    private JButton btnLisaa;
    private final JButton btnLisSarake = new JButton("Lis\u00E4\u00E4 sarake");
    private JLabel labelVirhe;
    /** rivi jolla ollaan */
    protected JLabel labelRow;
    /** sarake jolla ollaan */
    protected JLabel labelColumn;


    /**
     * Launch the application.
     * @param args ei kytss
     */
    public static void main(String[] args) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Throwable e) {
            e.printStackTrace();
        }
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                try {
                    StringTableSuunniteltuKayttotapa frame = new StringTableSuunniteltuKayttotapa();
                    frame.setVisible(true);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }


    /**
     * Create the frame.
     */
    public StringTableSuunniteltuKayttotapa() {

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setBounds(100, 100, 500, 300);
        contentPane = new JPanel();
        contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
        contentPane.setLayout(new BorderLayout(0, 0));
        setContentPane(contentPane);

        table = new StringTable();
        table.addTableSelectionListener(new TableSelectionListener() {
            @Override
            public void selectionChanged(IStringTable sender, int row, int column) {
                labelRow.setText(""+row);
                labelColumn.setText(""+column);
            }
        });
        table.addTableErrorListener(new TableErrorListener() {
            @Override
            public void tableError(IStringTable sender, int row, int column, String virhe) {
                setError(sender,row,column,virhe);
            }
        });
        table.addTableEditListener(new TableEditListener() {
            @Override
            public String tableEdit(IStringTable sender, int row, int column, Object s) {
                return checkNewValue(sender,row,column,s);
            }
        });
        table.getTable().setRowSelectionAllowed(false);
        table.getTable().setModel(new DefaultTableModel(
            new Object[][] {
                {null, null, null},
                {null, new Integer(12), new Double(30.0)},
                {null, null, null},
                {null, null, null},
            },
            new String[] {
                "Nimi", "Nro", "Op"
            }
        ) {
            /**
             * 
             */
            private static final long serialVersionUID = 1L;
            @SuppressWarnings("rawtypes")
            private Class[] columnTypes = new Class[] {
                String.class, Integer.class, Double.class
            };
            @Override
            @SuppressWarnings({ "rawtypes", "unchecked" })
            public Class getColumnClass(int columnIndex) {
                if ( columnIndex < columnTypes.length )
                    return columnTypes[columnIndex];
                return Double.class;
            }
        });

        contentPane.add(table, BorderLayout.CENTER);
        
        labelVirhe = new JLabel("");
        labelVirhe.setOpaque(true);
        labelVirhe.setBackground(Color.RED);
        table.add(labelVirhe, BorderLayout.SOUTH);
        contentPane.add(panelValikot, BorderLayout.SOUTH);

        btnPoistaValittuRivi = new JButton("Poista valittu rivi");
        btnPoistaValittuRivi.addActionListener(new ActionListener() {
            @SuppressWarnings("synthetic-access")
            @Override
            public void actionPerformed(ActionEvent arg0) {
                table.removeSelectedRow();
            }
        });
        
        labelRow = new JLabel("0");
        panelValikot.add(labelRow);
        
        labelColumn = new JLabel("0");
        panelValikot.add(labelColumn);
        panelValikot.add(btnPoistaValittuRivi);

        btnHaeArvo = new JButton("Hae arvo");
        btnHaeArvo.addActionListener(new ActionListener() {
            @SuppressWarnings("synthetic-access")
            @Override
            public void actionPerformed(ActionEvent arg0) {
                JOptionPane.showMessageDialog(null,
                        table.getSelectedValue());
            }
        });
        panelValikot.add(btnHaeArvo);

        btnTulostaKaikki = new JButton("tulosta kaikki");
        btnTulostaKaikki.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) { //
            }
        });
        panelValikot.add(btnTulostaKaikki);
        
        btnLisaa = new JButton("Lis\u00E4\u00E4");
        btnLisaa.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                // table.addRow();
                lisaaOppilas();
            }
        });
        panelValikot.add(btnLisaa);
        btnLisSarake.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                lisaaSarake();
            }
        });
        
        panelValikot.add(btnLisSarake);
        
        alustaAlkiot();
        alkiotTaulukkoon();
    }


  

//====================================================================================
  // Itse kirjoitettavat metodit:    
  //====================================================================================

    private ArrayList<Opiskelija> lista = new ArrayList<Opiskelija>();

    /**
     * Listn uusi sarake
     */
    protected void lisaaSarake() {
        ((DefaultTableModel)table.getTable().getModel()).addColumn("hinta");
    }
    
    
   /**
     * Lis uuden opiskelijan taulukkoon
     */
    protected void lisaaOppilas() {
      lista.add(new Opiskelija());
      alkiotTaulukkoon();
    }


    /**
     * Asettaa virheen nkyville
     * @param sender milt taulukolta virhe tuli
     * @param row milt rivilt
     * @param column milt sarakkeelta
     * @param virhe virheen teksti 
     */
    @SuppressWarnings("unused")
    protected void setError(IStringTable sender, int row, int column, String virhe) {
        String s = virhe;
        if ( s == null ) s = "";
        labelVirhe.setText(s);
    }
    
    
    /**
     * Tarkistaa arvon sopivuuden
     * @param sender mist taulukosta pyynt tuli
     * @param row mit rivi pyynt koskee
     * @param column mit saraketta pyynt koskee
     * @param s mik jono halutaan asettaa
     * @return null jos asetus onnistuu, muuten virheilmoitus
     */
    protected String checkNewValue(IStringTable sender, int row, int column, Object s) {
        Opiskelija o = (Opiskelija)sender.getObjectAt(row);
        return o.setAttribute(column, s);
    }

    
    private void alustaAlkiot() {
        lista.add(new Opiskelija("Masi", 50, 0));
        lista.add(new Opiskelija("Kessu", 34, 50));
        lista.add(new Opiskelija("Aku", 1, 46));
        lista.add(new Opiskelija("Iines", 2, 46));
    }    
        

    private void alkiotTaulukkoon() {
        table.clear(); // tyhjennetn taulukko
        for (Opiskelija o : lista) { // kydn kaikki opiskelijat lvitse.
            int r = table.addRow();  // listn taulukkon uusi rivi
            table.setObjectAt(o, r); // Listn kullekin riville viite siit, mihin olioon rivi liittyy.
            //   table.setValueAt(o.getNimi(),r, 0 );           // voi viittailla suoraan gettereihin jos jaksaa
            table.setValueAt(o.getAttribute(0), r, 0); // tai kytt (oikeasti silmukassa) indeksoitua getteri
            table.setValueAt(o.getAttribute(1), r, 1); // asetetaan kunkin sarakkeen arvo omasta datasta
            table.setValueAt(o.getAttribute(2), r, 2);
        }
        table.setCellColor(2, 1, Color.YELLOW);  // Malliksi yksi solu keltaiseksi.
    }



    //====================================================================================
    // Dataluokka    
    //====================================================================================

    /**
     * @author Samu Tuominen
     * @version 5.4.2011
     */
    public static class Opiskelija {

        private String nimi;
        private int opiskelijanro;
        private double opintopisteita;


        /**
         * Luo tyhjn opiskelijan
         */
        public Opiskelija() {
            nimi = "";
        }


        /**
         * Palauttaa i:n attribuutin luokan
         * @param i monesko attribuutti
         * @return ':n attribuutin luokka
         */
        public Class<?> getClass(int i) {
            switch (i) {
                case 0:
                    return String.class;
                case 1:
                    return Integer.class;
                case 2:
                    return Double.class;
                default: // tm vain virheellist kytt varten
                    return String.class;
            }
        }


        /**
         * Palauttaa ':n attribuutin
         * @param i monesko attribuutti
         * @return i's attribuutti
         */
        public Object getAttribute(int i) {
            switch (i) {
                case 0:
                    return getNimi();
                case 1:
                    return opiskelijanro;
                case 2:
                    return opintopisteita;
                default: // tm vain virheellist kytt varten
                    return null;
            }
        }


        /**
         * Asettaa i'n attribuutin arvon
         * @param i monesko attribuutti asetetaan
         * @param attribute asetettava arvo
         * @return onnistuiko asettaminen, null = ok, muut virheilmoitus
         */
        public String setAttribute(int i, Object attribute) {
            switch (i) {
                case 0:
                    nimi = ((String)attribute);
                    return null;
                case 1:
                    opiskelijanro = ((Integer)attribute);
                    return null;
                case 2:
                    Double p = ((Double)attribute);
                    if ( p > 400 ) return "Ei saa olla yli 400 op";
                    opintopisteita = p;
                    return null;
                default: // tm vain virheellist kytt varten
                    return "Vr kentn indeksi";
            }
        }


        /**
         * Otsikko i:nnelle attribuutille
         * @param i monesko attribuutti otetaan
         * @return i's otsikko
         */
        public String getHeader(int i) {
            switch (i) {
                case 0:
                    return "nimi";
                case 1:
                    return "opiskelijanro";
                case 2:
                    return "opintopisteit";
                default: // tm vain virheellist kytt varten
                    return null;
            }
        }


        /**
         * @param kuka nimi
         * @param nro opiskelijanumero
         * @param pisteita opintopisteit
         */
        public Opiskelija(String kuka, int nro, double pisteita) {
            this.nimi = kuka;
            this.opiskelijanro = nro;
            this.opintopisteita = pisteita;
        }


        /**
         * @return attribuuttien mr
         */
        public static int getAttributeCount() {
            return 3;
        }


        /**
         * @return nimi opiskelijan nimi
         */
        public String getNimi() {
            return nimi;
        }


        @Override
        public String toString() {
            StringBuilder arvo = new StringBuilder();
            arvo.append(nimi).append('|');
            arvo.append(opiskelijanro).append('|');
            arvo.append(opintopisteita);
            return arvo.toString();
        }

    }

}
